;---------------------------------------------------------------------------
;
; bus.asm
;
; Include file for Fluke SB-Bus interface
;
; Assembling started at 25-4-1997
;
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
; ROUTINE    :  SEND_CHAR
; FUNCTION   :  Send character to SB-Bus
;---------------------------------------------------------------------------

SEND_CHAR       PHA                     Save used registers
                PHX
                LDX     SELECTED        Are we selected?
                CPX     #%1000.0000
                BNE     .RTS            No!
                LDX     SB_OUT_HEAD     Save character in the buffer
                AND     #%0111.1111     Strip end flag
                STA     OUTPUT_BUFFER,X
                INX                     Increment pointer
                TXA
                AND     #%0001.1111     Limit range
                STA     SB_OUT_HEAD
.RTS            PLX                     Restore used registers
                PLA
                RTS

;---------------------------------------------------------------------------
; ROUTINE    :  SEND_STRING
; FUNCTION   :  Send string of characters to the bus
;---------------------------------------------------------------------------

SEND_STRING     STX     TEXT_POINTER    Save text pointer
                STY     TEXT_POINTER+1
                LDA     SELECTED        Are we selected?
                CMP     #%1000.0000
                BNE     SS_RTS          No!
SEND_STRING_Z   LDA     (TEXT_POINTER)  Get next character
                PHP                     Save status
                JSR     SEND_CHAR       Send character to output buffer
                INC     TEXT_POINTER    Increment text pointer
                BNE     .NO_CARRY       No carry!
                INC     TEXT_POINTER+1
.NO_CARRY       PLP                     End of string
                BPL     SEND_STRING_Z   Not yet!
SS_RTS          RTS

;---------------------------------------------------------------------------
; ROUTINE  :  SEND_VALUE_IND
; FUNCTION :  Send value, pointed to by X to the SB-Bus
;             X points to value, Y points to propper DP position
;---------------------------------------------------------------------------

SEND_VALUE_IND  LDA     READ_FLAG       Save changed flag
                PHA
                LDA     0,X             Get low byte of value
                STA     DECONV           and save it
                LDA     1,X             Get high byte of value
                STA     DECONV+1         and save it too
                BPL     .POS            Number is positive!
                LDA     DECONV          Take 2s complement
                EOR     #$FF
                ADD     #1
                STA     DECONV
                LDA     DECONV+1
                EOR     #$FF
                ADC     #0
                STA     DECONV+1
                LDA     DECONV          See if overload situation
                CMP     #2000
                LDA     DECONV+1
                SBC     /2000
                LDA     DECONV+1
                BCC     .NEG_NOT_OL     No!
                LDA     #%1111.1111     Set overload situation
                STA     DECONV
                LDA     #%0000.0111
.NEG_NOT_OL     ORA     #%0000.1000     Set sign flag
                STA     DECONV+1
                BRA     .GET_DP

.POS            LDA     DECONV          See if overload situation
                CMP     #2000
                LDA     DECONV+1
                SBC     /2000
                BCC     .GET_DP         No! Get decimal pointer position
                LDA     #%1111.1111     Set overload situation
                STA     DECONV
                LDA     #%0000.0111
                STA     DECONV+1

.GET_DP         LDA     0,Y             Get DP position
                ASL
                ASL
                ASL
                ASL
                ORA     DECONV+1        Combine with high byte
                STA     DECONV+1
                PLA                     See if reading changed
                CMP     READ_FLAG
                BNE     SEND_VALUE_IND  Reread the value
                LDA     DECONV          Get value in registers
                LDX     DECONV+1

;---------------------------------------------------------------------------
; ROUTINE  :  SEND_VALUE
; FUNCTION :  Send meter value to SB-Bus
; ON ENTRY :  A is LSB, X is MSB
;---------------------------------------------------------------------------

SEND_VALUE      STA     DECONV          Save value
                TXA                     See if value is 0
                AND     #%0000.0111
                ORA     DECONV
                BNE     .NOT_0          It's not!
                TXA                     If it is 0, it may not be negative
                AND     #%1111.0111
                TAX
.NOT_0          TXA
                LSR                     Separate the decimal point
                LSR
                LSR
                LSR
                AND     #%0000.0011
                STA     DP_COUNT
                TXA
                AND     #%0000.1000     Send sign to SB-Bus if negative
                BEQ     .POSITIVE       It's positive!
                LDA     #'-'
                JSR     SEND_CHAR
.POSITIVE       TXA                     Save high part of value too
                AND     #%0000.0111
                STA     DECONV+1
                EOR     #%0000.0111     OL situation?
                BNE     .CONVERT        No! Convert normally
                LDA     DECONV
                INC
                BNE     .CONVERT        No! Convert normally
                LDA     #9999           Convert 9999 to display
                STA     DECONV
                LDA     /9999
                STA     DECONV+1

.CONVERT        CLC                     Clear leading 0 flag
                PHP
                LDA     DECONV          See if total value is 0
                ORA     DECONV+1
                BEQ     .LAST_DIG       It is 0, just print 0!
                LDY     #-2             Prepare table index
.DECADE_LOOP    LDX     #-1             Clear digit counter
                INY                     Increment table pointer
                INY
                CPY     #3*2            End of table?
                BCS     .LAST_DIG       Yes!
.LOOP           INX                     Subtract decade as often as
                LDA     DECONV           possible
                SUB     .TABLE,Y
                PHA
                LDA     DECONV+1
                SBC     .TABLE+1,Y
                BCC     .DECADE         It didn't go anymore!
                STA     DECONV+1        Save intermediate result
                PLA
                STA     DECONV
                BRA     .LOOP           Try again!
.DECADE         PLA                     Clear stack
                TXA                     Get decade counter
                BNE     .NO_ZERO        It's no zero
                PLP                     Still leading 0?
                PHP
                BCC     .DO_DP          Yes!
.NO_ZERO        PLP                     Clear leading 0 flag
                SEC
                PHP
                ORA     #'0'            Make it ASCII
                JSR     SEND_CHAR        and send it to the bus
.DO_DP          DEC     DP_COUNT        Need DP here?
                BNE     .DECADE_LOOP    No!
                PLP                     Still leading 0?
                BCS     .NOT_LEAD_DP    No!
                LDA     #'0'            Send 0 now!
                JSR     SEND_CHAR
                SEC                     Indicate no leading 0 anymore
.NOT_LEAD_DP    PHP
                LDA     #'.'            Send DP now
                JSR     SEND_CHAR
                BRA     .DECADE_LOOP    Do next digit
.LAST_DIG       PLP                     Clear stack
                LDA     DECONV          Send last character always
                ORA     #'0'
                JSR     SEND_CHAR
                RTS

.TABLE          .DA     1000            Decade table
                .DA     100
                .DA     10


;---------------------------------------------------------------------------
; ROUTINE  :  SEND_DEC
; FUNCTION :  Send a 16 bit decimal word the SB-Bus
; ON ENTRY :  A is LSB, X is MSB
;---------------------------------------------------------------------------

SEND_DEC        STA     DECONV          Save word
                STX     DECONV+1
                CLC                     Clear leading 0 flag
                PHP
                LDY     #-2             Setup index in table
.DECADE_LOOP    LDX     #-1             Clear digit counter
                INY                     Increment table pointer
                INY
                CPY     #4*2            End of table?
                BCS     .LAST_DIG       Yes!
.LOOP           INX                     Subtract decade as often as
                LDA     DECONV           possible
                SUB     .TABLE,Y
                PHA
                LDA     DECONV+1
                SBC     .TABLE+1,Y
                BCC     .DECADE         It didn't go anymore!
                STA     DECONV+1        Save intermediate result
                PLA
                STA     DECONV
                BRA     .LOOP           Try again!
.DECADE         PLA                     Clear stack
                TXA                     Get decade counter
                BNE     .NO_ZERO        It's no zero
                PLP                     Still leading 0?
                PHP
                BCC     .DECADE_LOOP    Yes!
.NO_ZERO        PLP                     Clear leading 0 flag
                SEC
                PHP
                ORA     #'0'            Make it ASCII
                JSR     SEND_CHAR        and send it to the bus
                BRA     .DECADE_LOOP    Do next digit
.LAST_DIG       PLP                     Clear stack
                LDA     DECONV          Send last character always
                ORA     #'0'
                JSR     SEND_CHAR
                RTS

.TABLE          .DA     10000           Decade table
                .DA     1000
                .DA     100
                .DA     10

;---------------------------------------------------------------------------
; ROUTINE  :  SEND_HEX
; FUNCTION :  Send a 16 bit hex word to the SB-Bus
; ON ENTRY :  A is LSB, X is MSB
;---------------------------------------------------------------------------

SEND_HEX        PHA                     Save low byte
                TXA                     Get high byte
                JSR     SEND_HEX_BYTE   Send it
                PLA                     Send low byte
SEND_HEX_BYTE   PHA
                LSR                     Send high nibble of this byte
                LSR
                LSR
                LSR
                JSR     .SEND_NIBBLE
                PLA
.SEND_NIBBLE    AND     #%0000.1111
                ORA     #'0'            Make it an ASCII sign
                CMP     #'9'+1
                BCC     .SKIP           It's a normal digit
                ADC     #7-1            Add 7 to it to start from A
.SKIP           JSR     SEND_CHAR
                RTS

;---------------------------------------------------------------------------
; ROUTINE  :  GET_DECIMAL
; FUNCTION :  Get a decimal number from input line
; UPON EXIT:  DECONV holds 16bit value, C=1 if overload,
;             Z=1 if nothing's input
;---------------------------------------------------------------------------

GET_DECIMAL     STZ     DECONV          Clear result
                STZ     DECONV+1
                CLC                     Clear nothing input flag
                PHP
                LDX     PARSE_SAVE
                JSR     NEXT_NON_SPACE
.LOOP           JSR     GET_UC          Get next character
                SUB     #'0'            Strip ASCII part
                CMP     #9+1            Was it a digit?
                BCS     .END            No! End of number found
                PLP                     Set digit found flag
                SEC
                PHP
                STA     DECONV+2        Save new digit
                ASL     DECONV          Multiply result by 10
                ROL     DECONV+1
                BCS     .OVERLOAD       We can't use overloads!
                LDA     DECONV
                LDY     DECONV+1
                ASL     DECONV
                ROL     DECONV+1
                BCS     .OVERLOAD       Again an overload situation!
                ASL     DECONV
                ROL     DECONV+1
                BCS     .OVERLOAD       Again an overload situation!
                ADC     DECONV
                STA     DECONV
                TYA
                ADC     DECONV+1
                STA     DECONV+1
                BCS     .OVERLOAD       Last posible overload for multiply!
                LDA     DECONV+2        Add new digit to result
                ADC     DECONV
                STA     DECONV
                LDA     DECONV+1
                ADC     #0
                STA     DECONV+1
                BCS     .OVERLOAD       Final overload posibility
                BRA     .LOOP           Do next digit!
.OVERLOAD       PLP                     Clear stack (auto C set!)
                RTS
.END            PLP                     Any digit found?
                BCC     .NODIGITS       No!
                STX     PARSE_SAVE
                LDA     #-1             Make Z=0
                CLC
                RTS
.NODIGITS       LDA     #0              Make Z=1
                CLC                     No overload error!
                RTS

;---------------------------------------------------------------------------
; ROUTINE  :  GET_HEX
; FUNCTION :  Get a hexa-decimal number from input line
; UPON EXIT:  DECONV holds 16bit value, C=1 if nothing's input
;             Z=1 if nothing's input, C=1 if overload
;---------------------------------------------------------------------------

GET_HEX         STZ     DECONV          Clear result
                STZ     DECONV+1
                CLC                     Clear nothing input flag
                PHP
                LDX     PARSE_SAVE      Get parse pointer
                JSR     NEXT_NON_SPACE
                JSR     GET_UC          First character must be $
                CMP     #'$'
                BNE     .NODIGITS_P     It's not!
.LOOP           JSR     GET_UC          Get next character
                SEC                     Strip ASCII part
                SBC     #'0'
                CMP     #9+1            Was it a digit?
                BCC     .09             A 0 to 9 digit found!
                SBC     #7              It can aslo be A to F
                CMP     #$F+1
                BCS     .END            It's not!
.09             STA     DECONV+2        Save new digit
                PLP                     Set digit found flag
                SEC
                PHP
                ASL     DECONV          Multiply result by 16
                ROL     DECONV+1
                BCS     .OVERLOAD       We can't use overloads!
                ASL     DECONV
                ROL     DECONV+1
                BCS     .OVERLOAD       Again an overload situation!
                ASL     DECONV
                ROL     DECONV+1
                BCS     .OVERLOAD       Again an overload situation!
                ASL     DECONV
                ROL     DECONV+1
                BCS     .OVERLOAD       Last posible overload for multiply!
                LDA     DECONV+2        Add new digit to result
                ADC     DECONV
                STA     DECONV
                LDA     DECONV+1
                ADC     #0
                STA     DECONV+1
                BCS     .OVERLOAD       Final overload posibility!
                BRA     .LOOP           Do next digit!
.OVERLOAD       PLP                     Clear stack (auto C set!)
                RTS
.END            PLP                     Any digit found?
                BCC     .NODIGITS       No!
                STA     PARSE_SAVE      Save parse pointer
                LDA     #-1             Make Z=0
                CLC
                RTS
.NODIGITS_P     PLA                     Clear stack
.NODIGITS       LDA     #0              Make Z=1
                CLC                     No overflow error!
                RTS

;---------------------------------------------------------------------------
; ROUTINE   :  POLL_SOUT
; FUNCTION  :  Poll serial output activity
;---------------------------------------------------------------------------

POLL_SOUT       LDA     SB_TRANSMIT     Transmitter active?
                BEQ     SOUT            No! See if there's a job to do
                RTS                     Let the interrupt handle it then

;---------------------------------------------------------------------------
; ROUTINE   :  SOUT
; FUNCTION  :  Handle serial output characters
;---------------------------------------------------------------------------

SOUT            LDA     SELECTED        Are we selected by own address?
                BPL     .EXIT           No! Not allowed to say something
                LDX     SB_OUT_TAIL     See if there's something in the
                CPX     SB_OUT_HEAD      output buffer
                BEQ     .EXIT           No! Forget it then
                LDA     S_HALTED        Serial output halted?
                BNE     .EXIT           Yes! Don't output
                LDA     CR_DELAY        Carriage return delay over?
                BNE     .EXIT           Not yet!

                LDA     OUTPUT_BUFFER,X Get output character
                STA     T2_ADDER         and send it out
                LDY     #9
                STY     T2_ACCU         Start transmission
                STY     SB_TRANSMIT     Make transmit flag <> 0
                STA     LAST_SEND
                CMP     #CR             Was it a CR that has been send?
                BNE     .NO_CR          No!
                LDA     FAST_MODE       Fast or slow mode?
                BEQ     .NO_CR          Fast mode! No need for a delay
                LDA     #2              Next char may only be transmitted
                STA     CR_DELAY         after at least 5ms
.NO_CR          INX                     Increment output pointer
                TXA
                AND     #%0001.1111     Limit value to 32
                STA     SB_OUT_TAIL     Save new value of buffer
                RTS
.EXIT           STZ     SB_TRANSMIT     Make transmit flag 0 to indicate
                RTS                      the idle situation

;---------------------------------------------------------------------------
; ROUTINE  :  STORE_IN
; FUNCTION :  Store new character to the input buffer if not full
;---------------------------------------------------------------------------

STORE_IN        PHA                     Save new character
                LDA     SB_IN_HEAD      Can buffer be full?
                CMP     SB_IN_TAIL
                BNE     .STORE          No! It can't be now
                LDA     CR_COUNT        It is if CR_COUNT > 0
                BEQ     .STORE          It's not fulle! Store it
                PLA                     Clear stack
                RTS
.STORE          LDA     SB_IN_HEAD      Increment input buffer pointer
                INC
                AND     #%0011.1111     Limit to 64 bytes
                CMP     SB_IN_TAIL      Buffer full?
                BEQ     .FULL           Yes!
                STA     SB_IN_HEAD      Save the new pointer value
                PLA                     Restore the character
                STA     INPUT_BUFFER,X
                INC     INLENGTH        Increment input length counter
                CMP     #CR             Was it CR?
                BNE     .RTS            No!
                INC     CR_COUNT        Increment the CR counter
                STZ     INLENGTH        Clear in length counter
.RTS            RTS
.FULL           PLA                     There's only room for one last CR
                CMP     #CR             Is this the terminating CR?
                BNE     .RTS            No! Simply forget this character
                STA     INPUT_BUFFER,X  Save this CR
                INC     CR_COUNT        Count it
                LDA     SB_IN_HEAD      Increment input buffer pointer
                INC
                AND     #%0011.1111     Limit to 64 bytes
                STA     SB_IN_HEAD      Save new character value
                RTS

;---------------------------------------------------------------------------
; ROUTINE  :  NEXT_NON_SPACE
; FUNCTION :  Let X point to next non space
; OUPUT    :  X=pointer to next non space
;             C=0 not end of line, C=1 end of line
;---------------------------------------------------------------------------

NEXT_NON_SPACE  LDA     INPUT_BUFFER,X  Get character
                CMP     #' '            Is it a space?
                BEQ     .SPACE          Yes!
                CMP     #CR             End of line?
                BEQ     .EOL            Yes!
                CLC                     Indicate not end of line
.EOL            RTS
.SPACE          INX                     Increment input buffer pointer
                TXA                     Limit to 64 bytes
                AND     #%0011.1111
                TAX
                BRA     NEXT_NON_SPACE  Try it again

;---------------------------------------------------------------------------
; ROUTINE  :  GET_UC
; FUNCTION :  Get next upper case character and increment pointer
; OUPUT    :  A=This upper-case character
;             X=Pointer to next character
;             Z=1 if blank or end of line
;---------------------------------------------------------------------------

GET_UC          LDA     INPUT_BUFFER,X  Get character
                CMP     #CR             Don't increment pointer if it's CR
                BEQ     .FORGET_CR
                PHA
                INX                     Increment pointer
                TXA
                AND     #%0011.1111      (limit to 64 bytes)
                TAX
                PLA
.FORGET_CR      CMP     #'a'            Is it lower case?
                BCC     .NO_LC          No!
                CMP     #'z'+1
                BCS     .NO_LC          No!
                AND     #%1101.1111     Make it upper case
.NO_LC          STA     LAST_CHAR       Save last character
                CMP     #' '            Make Z=1 if it's a space
                BEQ     .RTS            It is!
                CMP     #CR             Or a CR
.RTS            RTS

;---------------------------------------------------------------------------
; ROUTINE  :  DEC_PARSE
; FUNCTION :  Decrement parse pointer
;---------------------------------------------------------------------------

DEC_PARSE       LDA     LAST_CHAR       Was last character a CR?
                CMP     #CR
                BEQ     .RTS            Don't decrement then!
                DEX                     Decrement parse pointer and save it
                STX     PARSE_SAVE
.RTS            RTS

;---------------------------------------------------------------------------
; ROUTINE  :  SEND_STATUS
; FUNCTION :  Send status string (=>, ?> or !>), depending on last error
;---------------------------------------------------------------------------

SEND_STATUS     LDX     #.OK_TEXT       In case of no error
                LDY     /.OK_TEXT
                LDA     ERROR           What is the last error number?
                BEQ     .THIS_ONE       No error! Send =>
                LDX     #.SYN_TEXT      In case of syntax error
                LDY     /.SYN_TEXT
                DEC
                BEQ     .THIS_ONE       It was syntax error! Send ?>
                LDX     #.EXE_TEXT      Execution error, send !>
                LDY     /.EXE_TEXT
.THIS_ONE       JSR     SEND_STRING     Send this string
                JSR     CLR_SIN         Clear string from input buffer
                JMP     MAIN            Dirty return to main routine

.OK_TEXT        .AT     /=>/,#CR
.SYN_TEXT       .AT     /?>/,#CR
.EXE_TEXT       .AT     /!>/,#CR

;---------------------------------------------------------------------------
; ROUTINE  :  CLR_SIN
; FUNCTION :  Clear current string from serial input buffer
;---------------------------------------------------------------------------

CLR_SIN         LDA     CR_COUNT        Is there a string in the buffer?
                BEQ     .RTS            No!
                LDX     SB_IN_TAIL      Find end of string
.CLR_LOOP       LDA     INPUT_BUFFER,X  Get character
                PHA
                INX
                TXA
                AND     #%0011.1111     Limit to 64 bytes
                TAX
                PLA
                CMP     #CR
                BNE     .CLR_LOOP       Not end of line yet!
                DEC     CR_COUNT        Decrement CR counter
                STX     SB_IN_TAIL
.RTS            RTS

;---------------------------------------------------------------------------
; ROUTINE  :  POLL_SB
; FUNCTION :  Poll SB-Bus input status and decode if command completed
;---------------------------------------------------------------------------

POLL_SB         LDA     CR_COUNT        Has a complete command arrived yet?
                BNE     .COMMAND_IN     Yes it has!
                RTS
.COMMAND_IN     LDX     SB_IN_TAIL      Get tail pointer
                JSR     NEXT_NON_SPACE  Find first non space character
                BCS     .REPEAT         Repeat command!
                CMP     #'*'            Is it a system command?
                BEQ     .SYS_ALWAYS     Yes! Always allowed
                LDA     SELECTED        Are we selected by general call?
                BPL     .CLR_SIN        Yes! No normal commands allowed
.SYS_ALWAYS     LDA     #COMMAND_TABLE  Setup command table pointer
                LDY     /COMMAND_TABLE
                JSR     PARSE           Start parsing
                BCC     .SYNTAX_ERROR   Exit with syntax error!
                LDY     LAST_CHAR       Last character may not be a comma
                CPY     #','
                BEQ     .SYNTAX_ERROR   It is!
                ASL                     Multiply string number by 2
                TAX                      and use it as index
                LDA     CMD_ADDRESSES,X Get command address and add an
                STA     PARSE_POINTER    offset of 3 to get the repeat
                ADD     #3               command address
                STA     REPEAT_CMD
                LDA     CMD_ADDRESSES+1,X
                STA     PARSE_POINTER+1
                ADC     #0
                STA     REPEAT_CMD+1
                LDA     FAST_MODE       Slow mode active?
                BEQ     .FAST           No!
                LDA     #2              Delay at least 5ms
                STA     CR_DELAY
.FAST           JMP     (PARSE_POINTER) Execute the command
.REPEAT         LDA     ERROR           Was it syntax error last time?
                CMP     #1
                BEQ     .SYNTAX_ERROR   Yes! Than it will be this time too
                JMP     (REPEAT_CMD)    Repeat last command
.CLR_SIN        JMP     CLR_SIN         Clear this line from buffer
.SYNTAX_ERROR   LDA     #1              Syntax error
                STA     ERROR
                JMP     SEND_STATUS     Send syntax error status

;---------------------------------------------------------------------------
; ROUTINE  :  PARSE
; FUNCTION :  Try to parse a string
; INPUT    :  A=Parse table pointer L
;             Y=Parse table pointer H
;             X=Parse string begin
; OUTPUT   :  A=String number (Starting from 0)
;             C=0 string not found, C=1 string is found
;             PARSE_SAVE=Begin of string if C=0, end of string if C=1
;---------------------------------------------------------------------------

PARSE           STA     PARSE_POINTER   Save parse pointer
                STY     PARSE_POINTER+1
                STX     PARSE_SAVE      Save parse string begin pointer
                STZ     PARSE_COUNTER   Clear parse counter
.TRY_NEXT       LDX     PARSE_SAVE      Restore parse string begin
.LOOP           LDA     (PARSE_POINTER) End of parse table?
                BEQ     .NOT_FOUND      Yes! Not found!
                JSR     GET_UC          Get an upper case character
                BEQ     .NOT_THIS_ONE   Premature end of parse string!
                CMP     #','            A comma is end too
                BEQ     .NOT_THIS_ONE   It is a comma!
                EOR     (PARSE_POINTER) Compare with parse table
                ASL                     Ignore end flag
                BNE     .NOT_THIS_ONE   It's not this one!
                BCS     .LAST_CHAR      It's the last character!
                INC     PARSE_POINTER   Increment parse pointer
                BNE     .LOOP           No overflow!
                INC     PARSE_POINTER+1
                BRA     .LOOP
.LAST_CHAR      JSR     GET_UC          Next character must be a blank
                BEQ     .FOUND_STRING   It is! Found the string!
                CMP     #','            Or a comma
                BEQ     .FOUND_STRING   It is!
                INC     PARSE_COUNTER   Increment string counter
                INC     PARSE_POINTER   Point to begin of next string
                BNE     .TRY_NEXT       No carry!
                INC     PARSE_POINTER+1
                BRA     .TRY_NEXT
.NOT_THIS_ONE   INC     PARSE_COUNTER   Increment string counter
.FIND_END       LDA     (PARSE_POINTER) Is this the end already?
                ASL
                INC     PARSE_POINTER   Increment parse pointer and
                BNE     .NO_CARRY        don't change the real carry
                INC     PARSE_POINTER+1
.NO_CARRY       BCC     .FIND_END       This is not the end yet!
                BRA     .TRY_NEXT       Try to match next table entry
.NOT_FOUND      CLC                     Indicate string not found
                RTS
.FOUND_STRING   STX     PARSE_SAVE      Save new last parse pointer
                LDA     PARSE_COUNTER   Get string number
                SEC                     Indicate string is found
                RTS

;---------------------------------------------------------------------------
; TABLE    :  COMMAND_TABLE
; FUNCTION :  Table with all SB-Bus commands
;---------------------------------------------------------------------------

COMMAND_TABLE   .AT     /*CATALOG?/
                .AT     /*ERROR?/
                .AT     /*FAST/
                .AT     /*FLOW/
                .AT     /*FLOW?/
                .AT     /*HOLD/
                .AT     /*ID?/
                .AT     /*LOCS/
                .AT     /*REMS/
                .AT     /*RST/
                .AT     /*SLAVE/
                .AT     /*SLOW/
                .AT     /*TRIG/
                .AT     /*TST?/
                .AT     /AVG?/
                .AT     /CLEAR/
                .AT     /DUMP?/
                .AT     /HOLD/
                .AT     /INTERVAL/
                .AT     /INTERVAL?/
                .AT     /LIST?/
                .AT     /MAX?/
                .AT     /MEAN?/
                .AT     /MIN?/
                .AT     /OPTION/
                .AT     /READ?/
                .AT     /SAMPLES?/
                .AT     /START/
                .AT     /STATUS?/
                .AT     /STOP/
                .DA     #0

;---------------------------------------------------------------------------
; TABLE    :  CMD_ADDRESSES
; FUNCTION :  Pointers to command routines
;---------------------------------------------------------------------------

CMD_ADDRESSES   .DA     CATALOG_REQ     *CATALOG?
                .DA     ERROR_REQ       *ERROR?
                .DA     FAST_CMD        *FAST
                .DA     FLOW_CMD        *FLOW
                .DA     FLOW_REQ        *FLOW?
                .DA     HOLD_CMD        *HOLD
                .DA     ID_REQ          *ID?
                .DA     LOCS_CMD        *LOCS
                .DA     REMS_CMD        *REMS
                .DA     RST_CMD         *RST
                .DA     SLAVE_CMD       *SLAVE
                .DA     SLOW_CMD        *SLOW
                .DA     TRIG_CMD        *TRIG
                .DA     TST_REQ         *TST?
                .DA     MEAN_REQ        AVG?
                .DA     CLEAR_CMD       CLEAR
                .DA     DUMP_REQ        DUMP?
                .DA     HOLDREAD_CMD    HOLD
                .DA     INTERVAL_CMD    INTERVAL
                .DA     INTERVAL_REQ    INTERVAL?
                .DA     LIST_REQ        LIST?
                .DA     MAX_REQ         MAX?
                .DA     MEAN_REQ        MEAN?
                .DA     MIN_REQ         MIN?
                .DA     OPTION_CMD      OPTION
                .DA     READ_REQ        READ?
                .DA     SAMPLES_REQ     SAMPLES?
                .DA     START_CMD       START
                .DA     STATUS_REQ      STATUS?
                .DA     STOP_CMD        STOP

;---------------------------------------------------------------------------
; ROUTINE  :  TEST_HOLD
; FUNCTION :  Test HOLD mode status
;---------------------------------------------------------------------------

TEST_HOLD       LDA     HOLD_ACTIVE     Is hold mode active?
                BEQ     .RTS            No!
                LDA     HOLD_FUNCTION   Is a hold function loaded yet?
                ORA     HOLD_FUNCTION+1
                BNE     .HOLD_ERROR     Yes! Error! Can't have 2 in hold
                PLA                     Don't execute the command now but
                STA     HOLD_FUNCTION    put it's address in the hold
                PLA                      function
                STA     HOLD_FUNCTION+1
                INC     HOLD_FUNCTION   Increment function address by 1
                BNE     .NO_CARRY       No carry!
                INC     HOLD_FUNCTION+1
.NO_CARRY       JMP     SEND_STATUS     Exit without error
.HOLD_ERROR     LDA     #9              Exit with hold mode deactivated
                STA     ERROR            error
                STZ     HOLD_ACTIVE     Clear hold mode
                STZ     HOLD_FUNCTION
                STZ     HOLD_FUNCTION+1
                JMP     SEND_STATUS
.RTS            RTS

;---------------------------------------------------------------------------
; ROUTINE  :  NOT_GERERAL
; FUNCTION :  Don't do anything if selected by general call
;---------------------------------------------------------------------------

NOT_GENERAL     LDA     SELECTED        How are we selected?
                BPL     .GENERAL        By general call!
                RTS                     Do this command
.GENERAL        JMP     SEND_STATUS     Don't do this command

;---------------------------------------------------------------------------
; ROUTINE  :  TEST_ERROR
; FUNCTION :  Test last error results
;---------------------------------------------------------------------------

TEST_ERROR      LDA     ERROR           Did an error occur?
                BEQ     .RTS            No!
                CMP     #3              Aborted error is allowed
                BEQ     .RTS            It is!
                CMP     #8              As is too many errors
                BEQ     .RTS
                JMP     SEND_STATUS     Send error status again
.RTS            STZ     ERROR           Clear last error
                RTS

;---------------------------------------------------------------------------
; ROUTINE  :  NO_PARAMETERS
; FUNCTION :  See if really no parameters are given
;---------------------------------------------------------------------------

NO_PARAMETERS   STZ     ERROR           Clear error
                LDX     PARSE_SAVE      See if anything follows
                JSR     NEXT_NON_SPACE
                BCS     .RTS            No! O.K.!
                LDA     #4              Exit with parameters not allowed
                STA     ERROR            error
                JMP     SEND_STATUS
.RTS            RTS

;---------------------------------------------------------------------------
; ROUTINE  :  NO_MORE_PARS
; FUNCTION :  No more parameters allowed
;---------------------------------------------------------------------------

NO_MORE_PARS    STZ     ERROR           Start without an error
                LDA     LAST_CHAR       What was the last character?
                CMP     #CR             End of string?
                BEQ     .RTS            Yes!
                CMP     #','            Was it a comma?
                BEQ     .TOO_MANY       Yes! So there's more comming
                CMP     #' '            Was it something else than a space?
                BNE     .ILLEGAL        Yes! That's illegal
                JSR     NEXT_NON_SPACE  See if more follows a space
                BCS     .RTS            No! O.K.!
                LDA     #7              Exit with illegal parameter
                STA     ERROR
                JMP     SEND_STATUS
.TOO_MANY       LDA     #6              Exit with too many parameters error
                STA     ERROR
                JMP     SEND_STATUS
.ILLEGAL        LDA     #7              Exit with illegal parameter error
                STA     ERROR
                JMP     SEND_STATUS
.RTS            RTS

;---------------------------------------------------------------------------
; ROUTINE  :  ACK_WAIT
; FUNCTION :  Wait for acknowledge if it's enabled
;---------------------------------------------------------------------------

ACK_WAIT        LDA     FLOW_MODE       What type of flow control is active
                BEQ     .XOFF           Xon/Xoff, so don't clear input bffr
.CLR_SIN        JSR     CLR_SIN         Clear string from input buffer
                BNE     .CLR_SIN        Not all strings done!
.XOFF           LDA     STOP_ACT        Stop all activities?
                BNE     QUIT            Yes!
                PHY                     Don't let them tough this
                JSR     DO_EVENTS       Service other routines
                PLY
                LDA     SB_OUT_TAIL     Output buffer empty yet?
                CMP     SB_OUT_HEAD
                BNE     .XOFF           Not yet!
                LDA     FLOW_MODE       What flow control mode is active?
                BNE     .ACKNOWLEDGE    Acknowledge flow control!
                SEC                     Indicate no error
                RTS
.ACKNOWLEDGE    PHY                     Save this
                JSR     DO_EVENTS       Continue polling other functions
                PLY
                LDA     CR_COUNT        Is a CR received already?
                BEQ     .ACKNOWLEDGE    Not Yet!
                LDX     SB_IN_TAIL      Get pointer in input buffer
                JSR     NEXT_NON_SPACE  Get 1st non space character
                BCS     .ABORT          No acknowledge! Abort
                JSR     GET_UC          Get this 1st character
                CMP     #'='            OK status?
                BEQ     .OK             Yes!
                CMP     #'!'            Other legal character?
                BEQ     .ACK_ERR        Yes! Acknowledge error
                CMP     #'?'
                BNE     .ABORT          No! Abort the function
.ACK_ERR        JSR     NEXT_NON_SPACE  No more characters allowed
                BCC     .ABORT          There are more!
                INC     ERROR_COUNT     Increment acknowledge error
                LDA     ERROR_COUNT     Maximum number of errors found?
                CMP     #MAX_ACK_ERRORS
                BCS     .MAX_ERROR      Yes! Abort transfer
                CLC                     Indicate that string must
                RTS                      be repeated
.OK             JSR     NEXT_NON_SPACE  No more characters allowed
                BCC     .ABORT          There are more!
                STZ     ERROR_COUNT     Restore error counter
                SEC                     Indicate that everything's O.K.
                RTS
.ABORT          LDA     #3              Exit with aborted error
                STA     ERROR
                JMP     SEND_STATUS
.MAX_ERROR      LDA     #8              Exit with too many errors
                STA     ERROR
                JMP     SEND_STATUS

;---------------------------------------------------------------------------
; ROUTINE  :  QUIT
; FUNCTION :  Quit because of a stop activity command
;---------------------------------------------------------------------------

QUIT            LDA     SELECTED        Still selected?
                BPL     .MAIN           No! Nothing to do
                LDA     LAST_SEND       Stopped by a received address?
                BMI     .MAIN           Yes! Nothing to do
                STZ     SB_OUT_HEAD     Clear output buffer
                STZ     SB_OUT_TAIL
                STZ     SB_OUT_HEAD     Do this twice in case an interrupt
                STZ     SB_OUT_TAIL      has changedit again
                CMP     #CR             What was the last sent character?
                BEQ     .CR_DONE        It was a CR!
                LDA     #CR             Send an extra CR
                JSR     SEND_CHAR
.CR_DONE        LDA     #3              Exit with aborted error
                STA     ERROR
                JMP     SEND_STATUS     Quit everything
.MAIN           JMP     MAIN

;---------------------------------------------------------------------------
; ROUTINE  :  NOTHING_TO_REP
; FUNCTION :  There is nothing to repeat the very first time
;---------------------------------------------------------------------------

NOTHING_TO_REP  LDA     #2              Exit with nothing to repeat error
                STA     ERROR
                JMP     SEND_STATUS

;---------------------------------------------------------------------------
; ROUTINE  :  CATALOG_REQ
; FUNCTION :  Request command catalog
;---------------------------------------------------------------------------

CATALOG_REQ     JSR     NO_PARAMETERS   No parameters allowed here!
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                JSR     NOT_GENERAL     Don't do anything if general call
                LDA     #COMMAND_TABLE  Setup pointer to command table
                STA     TEXT_POINTER
                LDA     /COMMAND_TABLE
                STA     TEXT_POINTER+1
.CAT_LOOP       LDA     TEXT_POINTER    Save pointer in case we have to
                STA     PARSE_POINTER    repeat the string
                LDA     TEXT_POINTER+1
                STA     PARSE_POINTER+1
                LDY     #0              Clear error counter
                LDA     (TEXT_POINTER)  End of table?
                BEQ     .THE_END        Yes!
.RETRY          JSR     SEND_STRING_Z   Send the string
                LDA     #CR             Terminate string with CR
                JSR     SEND_CHAR
                JSR     ACK_WAIT        Wait for acknowledge
                BCS     .CAT_LOOP       Acknowledged OK!
                LDA     PARSE_POINTER   Repeat this string for it went wrong
                STA     TEXT_POINTER
                LDA     PARSE_POINTER+1
                STA     TEXT_POINTER+1
                BRA     .RETRY
.THE_END        JMP     SEND_STATUS     Exit without error

;---------------------------------------------------------------------------
; ROUTINE  :  ERROR_REQ
; FUNCTION :  Request last error
;---------------------------------------------------------------------------

ERROR_REQ       JSR     .PARAMETERS     Handle parameter differently
                JSR     TEST_ERROR      Test execution error
                NOP                     Skip HOLD test
                NOP
                NOP
                JSR     NOT_GENERAL     Don't do anything if general call
                LDX     PARAMETERS      Get last error number
                INX
                LDA     #ERROR_TEXT     Setup pointer to error text
                STA     TEXT_POINTER
                LDA     /ERROR_TEXT
                STA     TEXT_POINTER+1
.LOOP           DEX                     Need this string?
                BEQ     .FOUND_STRING   Yes!
.FIND_NEXT      LDA     (TEXT_POINTER)  Find end of this string
                INC     TEXT_POINTER
                BNE     .NO_CARRY
                INC     TEXT_POINTER+1
.NO_CARRY       ASL
                BCC     .FIND_NEXT      Not found yet!
                BRA     .LOOP           Try this one!
.FOUND_STRING   JSR     SEND_STRING_Z   Send this string, without loading
                JMP     SEND_STATUS
.PARAMETERS     LDA     ERROR           Save original error
                STA     PARAMETERS
                STZ     ERROR
                JMP     NO_PARAMETERS   No parameters allowed

;---------------------------------------------------------------------------
; ROUTINE  :  FAST_CMD
; FUNCTION :  Setup fast reply mode
;---------------------------------------------------------------------------

FAST_CMD        JSR     NO_PARAMETERS   No parameters allowed here!
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                STZ     FAST_MODE       Setup fast mode
                JMP     SEND_STATUS     Exit with no error

;---------------------------------------------------------------------------
; ROUTINE  :  FLOW_CMD
; FUNCTION :  Setup flow control mode
;---------------------------------------------------------------------------

FLOW_CMD        JSR     .PARAMETER      Get parameter
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                LDA     PARAMETERS      Get parameter and store it
                BEQ     .ZERO           Store 0
                LDA     #$FF            Store $FF if not 0
.ZERO           STA     FLOW_MODE
                JMP     SEND_STATUS

.PARAMETER      STZ     ERROR           Clear error flag
                LDX     PARSE_SAVE      Find begin of parameter
                JSR     NEXT_NON_SPACE
                BCS     .MISSING        Parameter is missing!
                LDA     #.PAR_TABLE     Setup begin of parameter table
                LDY     /.PAR_TABLE
                JSR     PARSE           Parse parameter
                BCC     .ILLEGAL        Illegal parameter error!
                STA     PARAMETERS      Save parameter
                JMP     NO_MORE_PARS    No more parameters are allowed!

.MISSING        LDA     #5              Exit with missing parameter error
                STA     ERROR
                JMP     SEND_STATUS

.ILLEGAL        LDA     #7              Exit with illegal parameter error
                STA     ERROR
                JMP     SEND_STATUS

.PAR_TABLE      .AT     /XOFF/
                .AT     /ACK/
                .DA     #0

;---------------------------------------------------------------------------
; ROUTINE  :  FLOW_REQ
; FUNCTION :  Request current flow control mode
;---------------------------------------------------------------------------

FLOW_REQ        JSR     NO_PARAMETERS   No parameters allowed here!
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                JSR     NOT_GENERAL     Don't do anything if general call
                LDX     #.XON_XOFF      In case of xon/xoff mode
                LDY     /.XON_XOFF
                LDA     FLOW_MODE
                BEQ     .IT_IS          It is xon/xoff mode!
                LDX     #.ACKNOWLEDGE   Then it must be acknowledge!
                LDY     /.ACKNOWLEDGE
.IT_IS          JSR     SEND_STRING     Send the string
                JMP     SEND_STATUS     Exit with no error

.XON_XOFF       .AT     \XON/XOFF\,#CR
.ACKNOWLEDGE    .AT     /ACKNOWLEDGE/,#CR

;---------------------------------------------------------------------------
; ROUTINE  :  HOLD_CMD
; FUNCTION :  Start hold mode
;---------------------------------------------------------------------------

HOLD_CMD        JSR     NO_PARAMETERS   No parameters allowed here!
                JSR     TEST_ERROR      Test previous errors
                LDA     SELECTED        Selected by general call?
                ASL
                BMI     .NO_ERROR       Yes! Ignore *HOLD command
                LDA     HOLD_ACTIVE     Is hold mode active already?
                BNE     .HOLD_ERROR     Yes! Can't activate it twice
                STZ     HOLD_FUNCTION   Clear hold function pointer
                STZ     HOLD_FUNCTION+1
                DEC     HOLD_ACTIVE     Switch hold mode on
.NO_ERROR       JMP     SEND_STATUS     Exit with no error

.HOLD_ERROR     LDA     #9              Exit with hold mode deactivated
                STA     ERROR            error
                STZ     HOLD_ACTIVE     Clear hold mode too
                STZ     HOLD_FUNCTION
                STZ     HOLD_FUNCTION+1
                JMP     SEND_STATUS

;---------------------------------------------------------------------------
; ROUTINE  :  ID_REQ
; FUNCTION :  Request ID
;---------------------------------------------------------------------------

ID_REQ          JSR     NO_PARAMETERS   No parameters allowed here!
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                JSR     NOT_GENERAL     Don't do anything if general call
                LDA     FLUKE_MODEL     Get model numer
                BNE     .8012           It's the 8012!
                >SAY    .ID_8010        Send 8010 ID text
                JMP     SEND_STATUS     Exit without error
.8012           >SAY    .ID_8012        Send 8012 ID text
                JMP     SEND_STATUS     Exit without error

.ID_8010        .AT     /FLUKE 8010 V1.2/,#CR
.ID_8012        .AT     /FLUKE 8012 V1.2/,#CR

;---------------------------------------------------------------------------
; ROUTINE  :  LOCS_CMD
; FUNCTION :  Setup local control mode
;---------------------------------------------------------------------------

LOCS_CMD        JSR     NO_PARAMETERS   No parameters allowed here!
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                STZ     LOCAL_MODE      Setup local mode
                JMP     SEND_STATUS     Exit with no error

;---------------------------------------------------------------------------
; ROUTINE  :  REMS_CMD
; FUNCTION :  Setup remote control mode
;---------------------------------------------------------------------------

REMS_CMD        JSR     NO_PARAMETERS   No parameters allowed here!
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                LDA     #-1             Setup remote mode
                STA     LOCAL_MODE
                JMP     SEND_STATUS     Exit with no error

;---------------------------------------------------------------------------
; ROUTINE  :  RST_CMD
; FUNCTION :  Restart the machine, do a warm start
;---------------------------------------------------------------------------

RST_CMD         JSR     NO_PARAMETERS   No parameters allowed here!
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                JMP     RESET           Do a system reset!

;---------------------------------------------------------------------------
; ROUTINE  :  SLAVE_CMD
; FUNCTION :  Setup new slave address, only during general call
;---------------------------------------------------------------------------

SLAVE_CMD       JSR     .PARAMETER      Get slave address from parameter
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                LDA     PARAMETERS      Setup new address
                STA     MY_ADDRESS
                AND     #%0111.1111
                ORA     FLUKE_MODEL     Combine byte with wluke model
                JSR     WRITE_NVM        and write it to the NVM
                JMP     SEND_STATUS     Exit with no error

.PARAMETER      STZ     ERROR           Clear error flag
                LDX     PARSE_SAVE      Get new slave address
                JSR     NEXT_NON_SPACE
                STX     PARSE_SAVE
                BCS     .MISSING        Missing parameter!
                JSR     GET_HEX         Is it a hex number?
                BCS     .RANGE          Overflow error!
                BNE     .CHECK_RANGE    It is a hex number, check range!
                JSR     GET_DECIMAL     Get a decimal number
                BCS     .RANGE          Overflow error!
                BEQ     .ILLEGAL        No digits given, illegal operand
.CHECK_RANGE    LDA     DECONV+1        High byte must be 0!
                BNE     .RANGE          It's not!
                LDA     DECONV          Make high bit of address 1
                ORA     #%1000.0000
                CMP     #129+1          Reserved addresses?
                BCC     .RANGE          Yes! Forget it
                CMP     #255
                BEQ     .RANGE          Yes!
                STA     PARAMETERS      Save new address
                JMP     NO_MORE_PARS    No more parameters are allowed!

.MISSING        LDA     #5              Exit with missing parameter error
                STA     ERROR
                JMP     SEND_STATUS

.ILLEGAL        LDA     #7              Exit with illegal parameter error
                STA     ERROR
                JMP     SEND_STATUS

.RANGE          LDA     #12             Exit with range error
                STA     ERROR
                JMP     SEND_STATUS

;---------------------------------------------------------------------------
; ROUTINE  :  SLOW_CMD
; FUNCTION :  Setup slow reply mode
;---------------------------------------------------------------------------

SLOW_CMD        JSR     NO_PARAMETERS   No parameters allowed here!
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                LDA     #-1             Setup slow mode
                STA     FAST_MODE
                JMP     SEND_STATUS     Exit with no error

;---------------------------------------------------------------------------
; ROUTINE  :  TRIG_CMD
; FUNCTION :  Trigger an in HOLD function
;---------------------------------------------------------------------------

TRIG_CMD        JSR     NO_PARAMETERS   No parameters allowed here!
                JSR     TEST_ERROR      Test previous errors
                LDA     HOLD_ACTIVE     Is hold mode active?
                BEQ     .NO_HOLD        Nothing in hold!
                LDA     HOLD_FUNCTION   Is hold function available?
                ORA     HOLD_FUNCTION+1
                BEQ     .HOLD_EMPTY     No! Nothing to do!
                STZ     HOLD_ACTIVE     Clear hold flag
                JMP     (HOLD_FUNCTION) Execute the function

.NO_HOLD        LDA     #10             Exit with hold mode not active
                STA     ERROR
                BRA     .CLR_HOLD

.HOLD_EMPTY     LDA     #11             Exit with nothing in hold error
                STA     ERROR

.CLR_HOLD       STZ     HOLD_ACTIVE     Clear hold mode
                STZ     HOLD_FUNCTION
                STZ     HOLD_FUNCTION+1
                JMP     SEND_STATUS

;---------------------------------------------------------------------------
; ROUTINE  :  TST_REQ
; FUNCTION :  System test request
;---------------------------------------------------------------------------

TST_REQ         JSR     NO_PARAMETERS   No parameters allowed here!
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                JSR     NOT_GENERAL     Don't do anything if general call

.BARK_AGAIN     LDA     DOG_BARKED      Send # of times the dog barked
                LDX     #0
                JSR     SEND_DEC        Print it decimal
                >SAY    .BARKED_TXT     Send legend string
                JSR     ACK_WAIT        Wait for acknowledge
                BCC     .BARK_AGAIN     Resend the text!
.MEM_AGAIN      LDA     EEPROM_ERRORS   Get EEPROM error flags
                BNE     .MEM_1          There's something wrong with it
                >SAY    .MEM_OK_TXT     Say that memory is OK
                BRA     .MEM_DONE
.MEM_1          DEC                     Memory fails?
                BNE     .MEM_2          No!
                >SAY    .MEM_FAIL_TXT   Say that memory fails
                BRA     .MEM_DONE
.MEM_2          DEC                     Memory write fails?
                BNE     .MEM_3          No!
                >SAY    .MEM_WRITE_TXT  Say that memory write fails
                BRA     .MEM_DONE
.MEM_3          >SAY    .MEM_LOST_TXT   Say that memory is lost
.MEM_DONE       JSR     ACK_WAIT        Wait for acknowledge
                BCC     .MEM_AGAIN
                STZ     DOG_BARKED      Clear counters for next time
                STZ     EEPROM_ERRORS   Clear eeprom errors
                JMP     SEND_STATUS     Exit with no error

.BARKED_TXT     .AT     / WATCHDOG RESETS/,#CR
.MEM_OK_TXT     .AT     /MEMORY OK/,#CR
.MEM_FAIL_TXT   .AT     /MEMORY FAILS/,#CR
.MEM_WRITE_TXT  .AT     /MEMORY WRITE FAILS/,#CR
.MEM_LOST_TXT   .AT     /MEMORY LOST/,#CR

;---------------------------------------------------------------------------
; ROUTINE  :  CLEAR_CMD
; FUNCTION :  Clear statistical values
;---------------------------------------------------------------------------

CLEAR_CMD       JSR     NO_PARAMETERS   No parameters allowed here!
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode

                LDA     #-1             Set clear statistics flag
                STA     CLEAR_STAT

                JMP     SEND_STATUS     Exit with no error

;---------------------------------------------------------------------------
; ROUTINE  :  DUMP_REQ
; FUNCTION :  Dump all samples to the SB-Bus, until user presses ESC
;---------------------------------------------------------------------------

DUMP_REQ        JSR     NO_PARAMETERS   No parameters allowed here!
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                JSR     NOT_GENERAL     Don't do anything if general call

.AGAIN          LDY     READ_FLAG       Get the current value
                LDA     CURRENT_VALUE
                LDX     CURRENT_VALUE+1
                CPY     READ_FLAG       Value changed in the mean time?
                BNE     .AGAIN          Yes! Read it again

                PHY
                JSR     SEND_VALUE      Send value to SB-Bus
                LDA     #CR
                JSR     SEND_CHAR
                PLY

.WAIT           JSR     ACK_WAIT        Mainly do other tasks
                CPY     READ_FLAG       Wait for the next reading
                BNE     .AGAIN          Ah, a new reading has arrived!
                LDX     SELECTED        Are we still selected?
                CPX     #%1000.0000
                BEQ     .WAIT           Yep!
                RTS

                JMP     SEND_STATUS     Exit with no error

;---------------------------------------------------------------------------
; ROUTINE  :  HOLDREAD_CMD
; FUNCTION :  Hold reading command
;---------------------------------------------------------------------------

HOLDREAD_CMD    JSR     NO_PARAMETERS   No parameters allowed here!
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode

                LDA     #-1             Set the hold flag
                STA     HOLD_READINGS

                JMP     SEND_STATUS     Exit with no error

;---------------------------------------------------------------------------
; ROUTINE  :  INTERVAL_CMD
; FUNCTION :  Set new interval setting
;---------------------------------------------------------------------------

INTERVAL_CMD    JSR     .PARAMETER      Get the time parameter
                JSR     .TEST_ERROR     Test previous errors
                JSR     TEST_HOLD       Test hold mode

                LDA     PARAMETERS      Set new interval time
                STA     INTERVAL_TIME
                LDA     PARAMETERS+1
                STA     INTERVAL_TIME+1

                JMP     SEND_STATUS     Send error status

.PARAMETER      STZ     ERROR           Start from scratch
                LDX     PARSE_SAVE      Find begin of parameter
                JSR     NEXT_NON_SPACE
                STX     PARSE_SAVE
                BCS     .MISSING        Missing parameter!
                JSR     GET_DECIMAL     Get a decimal number
                BCS     .RANGE          Overflow error!
                BEQ     .ILLEGAL        No digits given, illegal operand
                LDA     DECONV          Check range (max=3600sec=1hour)
                STA     PARAMETERS
                CMP     #3600+1
                LDA     DECONV+1
                STA     PARAMETERS+1
                SBC     /3600+1
                BCS     .RANGE          It's more than 3600!
                JMP     NO_MORE_PARS    No more parameters are allowed!

.TEST_ERROR     LDA     ERROR           See if LOG file was active last time
                BEQ     .TEST_LOG       No error, see if log is active now!
                CMP     #14
                BEQ     .TEST_LOG       Log was active! See if it's still act.
                JMP     TEST_ERROR      Jump to normal error test routine
.TEST_LOG       LDA     LOG_ACTIVE      Is log still active?
                BNE     .LOG_ACTIVE     Yes! Log is active
                STZ     ERROR           Clear error code now
                RTS
.LOG_ACTIVE     LDA     #14             Exit with log is active error
                STA     ERROR
                JMP     SEND_STATUS

.MISSING        LDA     #5              Exit with missing parameter error
                STA     ERROR
                JMP     SEND_STATUS

.ILLEGAL        LDA     #7              Exit with illegal parameter error
                STA     ERROR
                JMP     SEND_STATUS

.RANGE          LDA     #12             Exit with range error
                STA     ERROR
                JMP     SEND_STATUS

;---------------------------------------------------------------------------
; ROUTINE  :  INTERVAL_REQ
; FUNCTION :  Read current interval setting
;---------------------------------------------------------------------------

INTERVAL_REQ    JSR     NO_PARAMETERS   No parameters allowed
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                JSR     NOT_GENERAL     Don't do anything if general call

                LDA     INTERVAL_TIME   Send interval time to the bus
                LDX     INTERVAL_TIME+1
                JSR     SEND_DEC
                LDA     #CR             Followed by a CR
                JSR     SEND_CHAR

                JMP     SEND_STATUS     Exit with no error

;---------------------------------------------------------------------------
; ROUTINE  :  LIST_REQ
; FUNCTION :  List a range of the log file
;---------------------------------------------------------------------------

LIST_REQ        JSR     .PARAMETERS     Get the range of the list
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                JSR     NOT_GENERAL     Don't do anything if general call

                LDA     PARAMETERS      Setup begin of the range
                STA     TEXT_POINTER
                LDA     PARAMETERS+1
                STA     TEXT_POINTER+1
.MAIN_LOOP      LDY     READ_FLAG       In case value changed in mean time
                LDA     TEXT_POINTER    Beyond end of table?
                CMP     LOG_PNTR
                LDA     TEXT_POINTER+1
                SBC     LOG_PNTR+1
                BCS     .END            Yes!
                CPY     READ_FLAG       Did value change in the mean time?
                BNE     .MAIN_LOOP      Maybe! Try again
                LDA     PARAMETERS+2    Beyond end parameter?
                CMP     TEXT_POINTER
                LDA     PARAMETERS+3
                SBC     TEXT_POINTER+1
                BCC     .END            Yes! Stop listing
.ACK_AGAIN      LDA     TEXT_POINTER    Send current pointer value to the
                LDX     TEXT_POINTER+1   SB-Bus
                JSR     SEND_DEC
                LDA     #','
                JSR     SEND_CHAR
                JSR     .GET_VALUE      Get the value from the log file
                LDA     DECONV          Send the value to the SB-Bus
                LDX     DECONV+1
                JSR     SEND_VALUE
                LDA     #CR             Followed by a CR
                JSR     SEND_CHAR
                JSR     ACK_WAIT        Wait for acknowledge if neccessary
                BCC     .ACK_AGAIN      Resend this string!
                INC     TEXT_POINTER    Increment the table pointer
                BNE     .MAIN_LOOP      No carry to high byte
                INC     TEXT_POINTER+1
                BRA     .MAIN_LOOP      Do next value

.END            CPY     READ_FLAG       Did value change in the mean time?
                BNE     .MAIN_LOOP      Maybe! Try again
                JMP     SEND_STATUS     Exit with no error

.ABORT          LDA     #3              Exit with aborted error
                STA     ERROR
                JMP     SEND_STATUS

.PARAMETERS     STZ     ERROR           Clear error status to begin with
                STZ     PARAMETERS      Setup default parameters to
                STZ     PARAMETERS+1     list from 0 to 700
                LDA     #700
                STA     PARAMETERS+2
                LDA     /700
                STA     PARAMETERS+3
                LDX     PARSE_SAVE      Find begin of parameter
                JSR     NEXT_NON_SPACE
                STX     PARSE_SAVE
                BCS     .RTS            No parameters, use default!
                CMP     #','            Was it the separator?
                BEQ     .LIST_TO        Yes! Use default begin value
                JSR     GET_DECIMAL     Get a decimal number
                BCS     .RANGE          Overflow error!
                BEQ     .ILLEGAL        No digits given, illegal operand!
                JSR     .CHECK_RANGE    Legal range is 1..700
                LDA     DECONV          Save begin parameter
                STA     PARAMETERS
                LDA     DECONV+1
                STA     PARAMETERS+1
                LDA     LAST_CHAR       Is more following?
                CMP     #','
                BEQ     .SEPARATOR      Maybe
                LDA     PARAMETERS      Copy begin to end to get a range of 1
                STA     PARAMETERS+2
                LDA     PARAMETERS+1
                STA     PARAMETERS+3
                BRA     .EXIT_PAR
.LIST_TO        INX                     Accept the comma
.SEPARATOR      STX     PARSE_SAVE      Save parse pointer
                JSR     GET_UC          What's next character?
                PHP                     Save flags
                LDX     PARSE_SAVE      Restore last parse pointer
                PLP                     Restore flags
                BEQ     .EXIT_PAR       List to end of string
                JSR     GET_DECIMAL     Get 2nd decimal number if it's there
                BCS     .RANGE          Overflow error!
                BEQ     .ILLEGAL        No digits given, illegal operand!
                JSR     .CHECK_RANGE    Legal range is 1..700
                LDA     DECONV          Save end parameter
                STA     PARAMETERS+2
                LDA     DECONV+1
                STA     PARAMETERS+3
.END_PAR        LDA     PARAMETERS+2    Check order of range
                CMP     PARAMETERS
                LDA     PARAMETERS+3
                SBC     PARAMETERS+1
                BCS     .EXIT_PAR       Order is ok!
                LDA     PARAMETERS      Swap both digits
                LDX     PARAMETERS+2
                STA     PARAMETERS+2
                STX     PARAMETERS
                LDA     PARAMETERS+1
                LDX     PARAMETERS+3
                STA     PARAMETERS+3
                STX     PARAMETERS+1
.EXIT_PAR       JMP     NO_MORE_PARS    No more parameters

.CHECK_RANGE    LDA     DECONV          Legal range is from 0...700
                CMP     #700+1
                LDA     DECONV+1
                SBC     /700+1
                BCS     .RANGE          Out of range!
.RTS            RTS

.RANGE          LDA     #12             Exit with range error
                STA     ERROR
                JMP     SEND_STATUS

.ILLEGAL        LDA     #7              Exit with illegal parameter error
                STA     ERROR
                JMP     SEND_STATUS

.GET_VALUE      LDA     TEXT_POINTER    Read value 0?
                ORA     TEXT_POINTER+1
                BNE     .NOT_1ST        No!
                LDA     FIRST_VALUE     Get first value
                STA     DECONV
                LDA     FIRST_VALUE+1
                STA     DECONV+1
                BRA     .DO_DP          Handle DP position

.NOT_1ST        LDA     TEXT_POINTER+1  Copy and decrement pointer
                STA     TEMP_POINTER+1
                LDA     TEXT_POINTER    Decrement pointer
                BNE     .NO_BORROW      No borrow from high byte neccessary!
                DEC     TEMP_POINTER+1
.NO_BORROW      DEC
                STA     TEMP_POINTER
                LDA     TEMP_POINTER+1  Calculate group address
                LSR                      If ptr<552 : ptr/4*6+$0300
                STA     DECONV+1         If ptr>551 : (ptr-552)/4*6+$00E0
                LDA     TEMP_POINTER
                ROR
                AND     #%1111.1110     /4*2
                STA     DECONV
                LDA     TEMP_POINTER
                AND     #%1111.1100     /4*4
                ADD     DECONV
                STA     DECONV
                LDA     TEMP_POINTER+1
                ADC     DECONV+1
                STA     DECONV+1
                LDA     DECONV          See if group address >= 138*6
                CMP     #138*6
                LDA     DECONV+1
                SBC     /138*6
                BCS     .2ND            Use 2nd buffer area
                LDA     DECONV          Add begin address of 1st buffer to
                ADD     #MAIN_LOG_BUFFER  the current pointer
                STA     TEMP_POINTER
                LDA     DECONV+1
                ADC     /MAIN_LOG_BUFFER
                STA     TEMP_POINTER+1
                BRA     .SKIP
.2ND            LDA     DECONV          Add begin address of 1st buffer
                ADD     #-138*6+SUB_LOG_BUFFER  minus 138*6 to current pointer
                STA     TEMP_POINTER
                LDA     DECONV+1
                ADC     /-138*6+SUB_LOG_BUFFER
                STA     TEMP_POINTER+1

.SKIP           LDA     TEXT_POINTER    Get index in this group
                DEC
                AND     #%0000.0011     (4 12bit readings in each group)
                CMP     #%0000.0011     Is it the specially packed reading?
                BEQ     .SPECIAL        Yes!
                ASL                     Use this pointer as index
                TAY
                LDA     (TEMP_POINTER),Y    Get the value from the table
                STA     DECONV
                INY
                LDA     (TEMP_POINTER),Y    Get the high byte too
                AND     #%0000.1111     Strip the other reading's bits off
                STA     DECONV+1
.DO_DP          LDA     LOG_DP          Get log DP position
                ASL
                ASL
                ASL
                ASL
                ORA     DECONV+1        And combine with reading
                STA     DECONV+1
                RTS

.SPECIAL        LDY     #1
                LDA     (TEMP_POINTER),Y  Get LS nibble
                LSR
                LSR
                LSR
                LSR
                STA     DECONV
                INY
                INY
                LDA     (TEMP_POINTER),Y  Get middle nibble
                AND     #%1111.0000
                ORA     DECONV
                STA     DECONV
                INY
                INY
                LDA     (TEMP_POINTER),Y  Get MS nibble
                LSR
                LSR
                LSR
                LSR
                STA     DECONV+1
                BRA     .DO_DP          Set DP at position

;---------------------------------------------------------------------------
; ROUTINE  :  MAX_REQ
; FUNCTION :  Read maximum display value
;---------------------------------------------------------------------------

MAX_REQ         JSR     PARM_ABS_HOLD   Get abs,signed,hold parameters
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                JSR     NOT_GENERAL     Don't do anything if general call

                LDA     PARAMETERS+1    Hold values wanted?
                BNE     .HOLD           Yes!
                LDY     #LAST_DP        Setup pointer for DP
                LDX     #MAX_ABS        Setup pointer for absolute maximum
                LDA     PARAMETERS
                BEQ     .SEND           Use absolute value!
                LDX     #MAX_SIGN       Use signed value instead
                BRA     .SEND           Send the value

.HOLD           LDY     #HOLD_DP        Setup pointer for DP
                LDX     #HOLD_MAX_ABS   Setup pointer for signed maximum
                LDA     PARAMETERS
                BEQ     .SEND           Use absolute value
                LDX     #HOLD_MAX_SIGN  Use signed value instead

.SEND           JSR     SEND_VALUE_IND  Send value to SB-Bus
                LDA     #CR
                JSR     SEND_CHAR

                JMP     SEND_STATUS     Exit with no error

;---------------------------------------------------------------------------
; ROUTINE  :  MEAN_REQ
; FUNCTION :  Read mean display value
;---------------------------------------------------------------------------

MEAN_REQ        JSR     PARM_ABS_HOLD   Get abs,signed,hold parameters
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                JSR     NOT_GENERAL     Don't do anything if general call

                LDA     PARAMETERS+1    Hold or normal value
                BNE     .HOLD           Hold value!
                LDA     PARAMETERS      Absolute or signed value
                BNE     .NS_AGAIN       Signed!

.NA_AGAIN       LDY     READ_FLAG
                LDX     #4              Get normal absolute sum (5 bytes)
.NA_SUM         LDA     MEAN_ABS,X
                STA     DIV_A,X
                DEX
                BPL     .NA_SUM
                LDX     #2              Get normal counter (3 bytes)
.NA_COUNT       LDA     MEAN_COUNT,X
                STA     DIV_B,X
                DEX
                BPL     .NA_COUNT
                LDX     #LAST_DP        Get last DP (in X)
                JSR     .GET_DP
                CPY     READ_FLAG       Value changed?
                BNE     .NA_AGAIN
                BRA     .DIVIDE         Do the division

.NS_AGAIN       LDY     READ_FLAG
                LDX     #4              Get normal signed sum (5 bytes)
.NS_SUM         LDA     MEAN_SIGN,X
                STA     DIV_A,X
                DEX
                BPL     .NS_SUM
                LDX     #2              Get normal counter (3 bytes)
.NS_COUNT       LDA     MEAN_COUNT,X
                STA     DIV_B,X
                DEX
                BPL     .NS_COUNT
                LDX     #LAST_DP        Get last DP (in X)
                JSR     .GET_DP
                CPY     READ_FLAG       Value changed?
                BNE     .NS_AGAIN
                JSR     .CHECK_SIGN
                BRA     .DIVIDE         Do the division

.HOLD           LDA     PARAMETERS      Absolute or signed value
                BNE     .HS_AGAIN       Signed!

.HA_AGAIN       LDY     READ_FLAG
                LDX     #4              Get hold absolute sum (5 bytes)
.HA_SUM         LDA     HOLD_MEAN_ABS,X
                STA     DIV_A,X
                DEX
                BPL     .HA_SUM
                LDX     #2              Get hold counter (3 bytes)
.HA_COUNT       LDA     HOLD_MEAN_COUNT,X
                STA     DIV_B,X
                DEX
                BPL     .HA_COUNT
                LDX     #HOLD_DP        Get hold DP (in X)
                JSR     .GET_DP
                CPY     READ_FLAG       Value changed?
                BNE     .HA_AGAIN
                BRA     .DIVIDE         Do the division

.HS_AGAIN       LDY     READ_FLAG
                LDX     #4              Get hold signed sum (5 bytes)
.HS_SUM         LDA     HOLD_MEAN_SIGN,X
                STA     DIV_A,X
                DEX
                BPL     .HS_SUM
                LDX     #2              Get hold counter (3 bytes)
.HS_COUNT       LDA     HOLD_MEAN_COUNT,X
                STA     DIV_B,X
                DEX
                BPL     .HS_COUNT
                LDX     #HOLD_DP        Get hold DP (in X)
                JSR     .GET_DP
                CPY     READ_FLAG       Value changed?
                BNE     .HS_AGAIN
                JSR     .CHECK_SIGN
                BRA     .DIVIDE         Do the division

.DIVIDE         STZ     DIV_B+3         Clear MSBs if divider
                STZ     DIV_B+4

                STZ     DECONV          Clear result
                STZ     DECONV+1

                LDA     DIV_B           Divide by 0?
                ORA     DIV_B+1
                ORA     DIV_B+2
                BEQ     .DIV0           Yes!

                LDA     DIV_A           Is sum 0?
                ORA     DIV_A+1
                ORA     DIV_A+2
                ORA     DIV_A+3
                ORA     DIV_A+4
                BEQ     .SEND           Yes! Send result of 0

.JUSTIFY        LDA     DIV_A+4         Left justify sum
                BMI     .JUSTIFIED      Done!
                ASL     DIV_A           Shift sum 1 position left
                ROL     DIV_A+1
                ROL     DIV_A+2
                ROL     DIV_A+3
                ROL     DIV_A+4
                ASL     DIV_B           Do the same with divider
                ROL     DIV_B+1
                ROL     DIV_B+2
                ROL     DIV_B+3
                ROL     DIV_B+4
                BRA     .JUSTIFY        Try again

.DIV0           LDA     #13             Exit with divide by 0 error
                STA     ERROR
                JMP     SEND_STATUS

.JUSTIFIED      LDY     #0              Shift B left until justified
.SHIFT_B        LDA     DIV_B+4
                BMI     .DIVIDE_LOOP    Done if left bit is 1
                INY                     Count number of shifts
                ASL     DIV_B
                ROL     DIV_B+1
                ROL     DIV_B+2
                ROL     DIV_B+3
                ROL     DIV_B+4
                BRA     .SHIFT_B

.DIVIDE_LOOP    LDA     DIV_A           See if B goes in A
                CMP     DIV_B
                LDA     DIV_A+1
                SBC     DIV_B+1
                LDA     DIV_A+2
                SBC     DIV_B+2
                LDA     DIV_A+3
                SBC     DIV_B+3
                LDA     DIV_A+4
                SBC     DIV_B+4
                BCC     .NO_GO          It didn't go!
                LDA     DIV_A           Realy subtract it now
                SUB     DIV_B
                STA     DIV_A
                LDA     DIV_A+1
                SBC     DIV_B+1
                STA     DIV_A+1
                LDA     DIV_A+2
                SBC     DIV_B+2
                STA     DIV_A+2
                LDA     DIV_A+3
                SBC     DIV_B+3
                STA     DIV_A+3
                LDA     DIV_A+4
                SBC     DIV_B+4
                STA     DIV_A+4
.NO_GO          ROL     DECONV          Roll C into result
                ROL     DECONV+1
                LSR     DIV_B+4         Shift divider to the right
                ROR     DIV_B+3
                ROR     DIV_B+2
                ROR     DIV_B+1
                ROR     DIV_B
                DEY                     Decrement itteration counter
                BPL     .DIVIDE_LOOP    Do it again!

.SEND           LDA     DECONV          See if value is overload
                CMP     #2000
                LDA     DECONV+1
                SBC     /2000
                BCC     .NOT_OL         It's not overload
                LDA     #%1111.1111     Set overload situation
                STA     DECONV
                LDA     #%0000.0111
                STA     DECONV+1

.NOT_OL         TXA                     Convert DP and sign with value
                ORA     DECONV+1
                TAX
                LDA     DECONV
                JSR     SEND_VALUE      Send value to SB-Bus
                LDA     #CR
                JSR     SEND_CHAR

                JMP     SEND_STATUS     Exit with no error

.GET_DP         LDA     0,X             Get DP positition
                ASL                     Move it the the propper bits
                ASL
                ASL
                ASL
                TAX                     And save it in X
                RTS

.CHECK_SIGN     LDA     DIV_A+4         Is the value negative?
                BPL     .HS_POS         No!
                LDA     DIV_A           Take 2s complement of value
                EOR     #$FF
                ADD     #1
                STA     DIV_A
                LDA     DIV_A+1
                EOR     #$FF
                ADC     #0
                STA     DIV_A+1
                LDA     DIV_A+2
                EOR     #$FF
                ADC     #0
                STA     DIV_A+2
                LDA     DIV_A+3
                EOR     #$FF
                ADC     #0
                STA     DIV_A+3
                LDA     DIV_A+4
                EOR     #$FF
                ADC     #0
                STA     DIV_A+4
                TXA                     Set sign flag
                ORA     #%0000.1000
                TAX
.HS_POS         RTS

;---------------------------------------------------------------------------
; ROUTINE  :  MIN_REQ
; FUNCTION :  Read minimum display value
;---------------------------------------------------------------------------

MIN_REQ         JSR     PARM_ABS_HOLD   Get abs,signed,hold parameters
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                JSR     NOT_GENERAL     Don't do anything if general call

                LDA     PARAMETERS+1    Hold values wanted?
                BNE     .HOLD           Yes!
                LDY     #LAST_DP        Setup pointer for DP
                LDX     #MIN_ABS        Setup pointer for absolute minimum
                LDA     PARAMETERS
                BEQ     .SEND           Use absolute value!
                LDX     #MIN_SIGN       Use signed value instead
                BRA     .SEND           Send the value

.HOLD           LDY     #HOLD_DP        Setup pointer for DP
                LDX     #HOLD_MIN_ABS   Setup pointer for signed minimum
                LDA     PARAMETERS
                BEQ     .SEND           Use absolute value
                LDX     #HOLD_MIN_SIGN  Use signed value instead

.SEND           JSR     SEND_VALUE_IND  Send value to SB-Bus
                LDA     #CR
                JSR     SEND_CHAR

                JMP     SEND_STATUS     Exit with no error

;---------------------------------------------------------------------------
; ROUTINE  :  OPTION_CMD
; FUNCTION :  Setup Fluke model option
;---------------------------------------------------------------------------

OPTION_CMD      JSR     .PARAMETER      Get parameter
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                LDA     PARAMETERS      Get parameter and store it
                BEQ     .8010           It's an 8010
                LDA     #%1000.0000     Make it an 8012
.8010           STA     FLUKE_MODEL
                LDA     MY_ADDRESS      Combine model with SB-Bus address
                AND     #%0111.1111
                ORA     FLUKE_MODEL
                JSR     WRITE_NVM       Save it in EEPROM
                JMP     SEND_STATUS

.PARAMETER      STZ     ERROR           Clear error flag
                LDX     PARSE_SAVE      Find begin of parameter
                JSR     NEXT_NON_SPACE
                BCS     .MISSING        Parameter is missing!
                LDA     #.PAR_TABLE     Setup begin of parameter table
                LDY     /.PAR_TABLE
                JSR     PARSE           Parse parameter
                BCC     .ILLEGAL        Illegal parameter error!
                STA     PARAMETERS      Save parameter
                JMP     NO_MORE_PARS    No more parameters are allowed!

.MISSING        LDA     #5              Exit with missing parameter error
                STA     ERROR
                JMP     SEND_STATUS

.ILLEGAL        LDA     #7              Exit with illegal parameter error
                STA     ERROR
                JMP     SEND_STATUS

.PAR_TABLE      .AT     /8010/
                .AT     /8012/
                .DA     #0

;---------------------------------------------------------------------------
; ROUTINE  :  READ_REQ
; FUNCTION :  Read current display value
;---------------------------------------------------------------------------

READ_REQ        JSR     PARM_HOLD       Get hold parameter
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                JSR     NOT_GENERAL     Don't do anything if general call

                LDA     PARAMETERS+1    Hold or normal value
                BNE     .AGAIN_HOLD     Hold value!
.AGAIN_NORM     LDY     READ_FLAG       Get the current value
                LDA     CURRENT_VALUE
                LDX     CURRENT_VALUE+1
                CPY     READ_FLAG       Value changed in the mean time?
                BNE     .AGAIN_NORM     Yes! Read it again
                BRA     .SEND

.AGAIN_HOLD     LDY     READ_FLAG       Get hold value
                LDA     HOLD_VALUE
                LDX     HOLD_VALUE+1
                CPY     READ_FLAG       Value changed in the mean time?
                BNE     .AGAIN_HOLD     Yes! Read it again

.SEND           JSR     SEND_VALUE      Send value to SB-Bus
                LDA     #CR
                JSR     SEND_CHAR

                JMP     SEND_STATUS     Exit with no error

;---------------------------------------------------------------------------
; ROUTINE  :  SAMPLES_REQ
; FUNCTION :  Request number of logged samples
;---------------------------------------------------------------------------

SAMPLES_REQ     JSR     NO_PARAMETERS   No parameters allowed here!
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                JSR     NOT_GENERAL     Don't do anyting if general selected

.AGAIN          LDY     READ_FLAG       In case number changed in mean time
                LDA     LOG_PNTR        Get current number of log samples
                LDX     LOG_PNTR+1
                CPY     READ_FLAG
                BNE     .AGAIN          Interrupt happened in the mean time!
                JSR     SEND_DEC        Send the number in decimal
                LDA     #CR             Followed by a CR
                JSR     SEND_CHAR

                JMP     SEND_STATUS     Exit with no error

;---------------------------------------------------------------------------
; ROUTINE  :  START_CMD
; FUNCTION :  Start the log mode
;---------------------------------------------------------------------------

START_CMD       JSR     .PARAMETER      Get Moment,Abs,Signed parameter
                JSR     .TEST_ERROR     Test previous errors
                JSR     TEST_HOLD       Test hold mode

.AGAIN          LDY     READ_FLAG       In case an interrupt occurs
                STZ     LOG_PNTR        Clear log pointer
                STZ     LOG_PNTR+1
                LDA     LAST_DP         Copy last DP position to Log DP
                STA     LOG_DP           position
                LDX     #7              Clear the log mean registers
.CLR_LOOP       STZ     LOG_MEAN_SUM,X
                DEX
                BPL     .CLR_LOOP
                LDA     READ_COUNTER    Synchronise logging to the reading
                STA     LOG_TIMER
                STZ     LOG_TIMER+1
                STZ     LOG_TIMER+2
                LDA     PARAMETERS      Save mode of operation and activate
                STA     LOG_ACTIVE       log mode
                CPY     READ_FLAG       Did an interrupt change anything?
                BNE     .AGAIN          Yes! Start again

                JMP     SEND_STATUS     Exit with no error

.PARAMETER      STZ     ERROR           Clear error flag
                LDA     #1              Default is sample mode
                STA     PARAMETERS
                LDX     PARSE_SAVE      Find begin of parameter
                JSR     NEXT_NON_SPACE
                STX     PARSE_SAVE
                BCS     .RTS            No parameters, use default
.LOOP           LDA     #.PAR_TABLE     Setup begin of parameter table
                LDY     /.PAR_TABLE
                JSR     PARSE           Parse parameter
                BCC     .ILLEGAL        Illegal parameter error!
                LSR                     Divide parameter by 2
                STA     PARAMETERS      Save parameter
.NO_PARM        INC     PARAMETERS      Parameters may not be 0
                JMP     NO_MORE_PARS    No more parameters are allowed!

.ILLEGAL        LDA     #7              Exit with illegal parameter error
                STA     ERROR
                JMP     SEND_STATUS

.TEST_ERROR     LDA     ERROR           See if log was already active
                BEQ     .TEST_LOG       No error, see if log is active now!
                CMP     #14
                BEQ     .TEST_LOG       Log was active! See if it's still act.
                JMP     TEST_ERROR      Test normal errors
.TEST_LOG       LDA     LOG_ACTIVE      Is log still active?
                BNE     .LOG_ACTIVE     Yes! Log is already active
                STZ     ERROR           Clear error code now
.RTS            RTS
.LOG_ACTIVE     LDA     #14             Set Log already active error
                STA     ERROR
                JMP     SEND_STATUS

.PAR_TABLE      .AT     /M/
                .AT     /MOMENT/
                .AT     /A/
                .AT     /ABS/
                .AT     /S/
                .AT     /SIGNED/
                .DA     #0

;---------------------------------------------------------------------------
; ROUTINE  :  STATUS_REQ
; FUNCTION :  Request current log status
;---------------------------------------------------------------------------

STATUS_REQ      JSR     NO_PARAMETERS   No parameters allowed here!
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode
                JSR     NOT_GENERAL     Don't do anyting if general selected
                LDA     LOG_ACTIVE      Get current log status
                ASL
                TAX                     Use as pointer to propper string
                LDA     .PNT_TABLE+1,X
                TAY
                LDA     .PNT_TABLE,X
                TAX
                JSR     SEND_STRING     Send this string

                JMP     SEND_STATUS     Exit with no error

.PNT_TABLE      .DA     .NOT_ACT_TXT    Not active text
                .DA     .MOMENT_TXT     Momentary sample mode
                .DA     .ABS_TXT        Abs mode text
                .DA     .SIGN_TXT       Signed mode text

.NOT_ACT_TXT    .AT     /LOG MODE OFF/,#CR
.MOMENT_TXT     .AT     /MOMENTARY LOG MODE/,#CR
.ABS_TXT        .AT     /ABSOLUTE MEAN LOG MODE/,#CR
.SIGN_TXT       .AT     /SIGNED MEAN LOG MODE/,#CR

;---------------------------------------------------------------------------
; ROUTINE  :  STOP_CMD
; FUNCTION :  Stop the log mode
;---------------------------------------------------------------------------

STOP_CMD        JSR     NO_PARAMETERS   No parameters allowed here!
                JSR     TEST_ERROR      Test previous errors
                JSR     TEST_HOLD       Test hold mode

                LDA     LOG_ACTIVE      Is log mode active?
                BEQ     .LOG_NOT_ACTIVE No!
                STZ     LOG_ACTIVE      Deactivate the log

                JMP     SEND_STATUS     Exit with no error

.LOG_NOT_ACTIVE LDA     #15             Exit with no log active error
                STA     ERROR
                JMP     SEND_STATUS

;---------------------------------------------------------------------------
; ROUTINE  :  PARM_ABS_HOLD
; FUNCTION :  Get ABS,SIGNED,HOLD paramters
;---------------------------------------------------------------------------

PARM_ABS_HOLD   STZ     PARAMETERS+2    Indicate abs, and signed allowed
                BRA     PARM_AH
PARM_HOLD       LDA     #-1             Indicate only hold allowed!
                STA     PARAMETERS+2
PARM_AH         STZ     ERROR           Clear error flag
                STZ     PARAMETERS      Clear parameters (default=abs,
                STZ     PARAMETERS+1     and not hold value)
                LDX     PARSE_SAVE      Find begin of parameter
                JSR     NEXT_NON_SPACE
                BCS     .NO_PARM        No parameters, use default
.LOOP           LDA     #.PAR_TABLE     Setup begin of parameter table
                LDY     /.PAR_TABLE
                JSR     PARSE           Parse parameter
                BCC     .ILLEGAL        Illegal parameter error!
                LSR                     Divide by 2 for short paramters
                CMP     #2
                BEQ     .HOLD           Hold parameter!
                LDY     PARAMETERS+2    Abs and signed allowed?
                BNE     .ILLEGAL        No!
                STA     PARAMETERS      Save parameter abs,signed parameter
                BRA     .SKIP
.HOLD           STA     PARAMETERS+1    Set hold parameter
.SKIP           LDA     LAST_CHAR       Is next character a comma?
                CMP     #' '
                BEQ     .NO_MORE        Blank!
                CMP     #CR
                BEQ     .NO_MORE        End of line!
                CMP     #','            Must be a comma now!
                BNE     .ILLEGAL        It's not!
                STX     PARSE_SAVE      Parse next parameter
                BRA     .LOOP
.NO_MORE        JMP     NO_MORE_PARS    No more parameters are allowed!

.NO_PARM        RTS

.ILLEGAL        LDA     #7              Exit with illegal parameter error
                STA     ERROR
                JMP     SEND_STATUS

.PAR_TABLE      .AT     /A/
                .AT     /ABS/
                .AT     /S/
                .AT     /SIGNED/
                .AT     /H/
                .AT     /HOLD/
                .DA     #0

;---------------------------------------------------------------------------
; TABLE    :  Error messages
;---------------------------------------------------------------------------

ERROR_TEXT      .AT     /NO ERROR/,#CR                          Error 0
                .AT     /SYNTAX ERROR/,#CR                      Error 1
                .AT     /NOTHING TO REPEAT ERROR/,#CR           Error 2
                .AT     /ABORTED ERROR/,#CR                     Error 3
                .AT     /NO PARAMETERS ALLOWED/,#CR             Error 4
                .AT     /MISSING PARAMETER ERROR/,#CR           Error 5
                .AT     /TOO MANY PARAMETERS ERROR/,#CR         Error 6
                .AT     /ILLEGAL PARAMETER ERROR/,#CR           Error 7
                .AT     /TOO MANY ERRORS/,#CR                   Error 8
                .AT     /HOLD MODE DEACTIVATED/,#CR             Error 9
                .AT     /HOLD NOT ACTIVE ERROR/,#CR             Error 10
                .AT     /NOTHING IN HOLD ERROR/,#CR             Error 11
                .AT     /RANGE ERROR/,#CR                       Error 12
                .AT     /DIVIDE BY 0 ERROR/,#CR                 Error 13
                .AT     /LOG ACTIVE ERROR/,#CR                  Error 14
                .AT     /LOG NOT ACTIVE ERROR/,#CR              Error 15

