;---------------------------------------------------------------------------
;
; dec.asm
;
; Include file for Fluke SB-Bus interface
;
; Assembling started at 25-4-1997
;
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
; RAM memory declaration
;---------------------------------------------------------------------------

                .OR     $0000
                .DU
INPUT_BUFFER    .BS     64              SB-Bus input buffer
OUTPUT_BUFFER   .BS     32              SB-Bus output buffer

MY_ADDRESS      .BS     1               My own SB-Bus address
CR_DELAY        .BS     1               Carriage return delay counter
LAST_SEND       .BS     1               Last send character to SB-Bus
SB_IN_HEAD      .BS     1               SB-Bus input buffer head pointer
SB_IN_TAIL      .BS     1               SB-Bus input buffer tail pointer
SB_OUT_HEAD     .BS     1               SB-Bus output buffer head pointer
SB_OUT_TAIL     .BS     1               SB-Bus output buffer tail pointer
SELECTED        .BS     1               Selected flags
                                        ; b7=1 selected
                                        ; b6=1 general call
STOP_ACT        .BS     1               Stop activities if <> 0
S_HALTED        .BS     1               Serial interface halted if <> 0
FAST_MODE       .BS     1               Fast mode if 0, slow mode if <> 0
FLOW_MODE       .BS     1               Flow control mode, XOFF=0, ack <> 0
LOCAL_MODE      .BS     1               Local mode if 0 (not implemented)
SB_TRANSMIT     .BS     1               SB-Bus output busy if <> 0
INLENGTH        .BS     1               SB-Bus last input string length
CR_COUNT        .BS     1               Number of carriage returns counted
LAST_CHAR       .BS     1               Used in parsing input strings
FLUKE_MODEL     .BS     1               8010 if 0, 8012 if %1000.0000
ERROR           .BS     1               Last error number
ERROR_COUNT     .BS     1               Acknowledge error counter
HOLD_ACTIVE     .BS     1               Hold mode active if <> 0
HOLD_FUNCTION   .BS     2               Function in HOLD address if <> 0

TEXT_POINTER    .BS     2               Genaral purpose text pointer
PARSE_POINTER   .BS     2               Parse table pointer
TEMP_POINTER    .EQ     PARSE_POINTER   Temp pointer used by list
REPEAT_CMD      .BS     2               Pointer to last command routine
PARSE_COUNTER   .BS     1               Parse counter
PARSE_SAVE      .BS     1               Begin of parse string save
EEPROM_ERRORS   .BS     1               IIC and write errors for EEPROM
;                                       0 = all's well
;                                       1 = no acknowledge
;                                       2 = write error
;                                       3 = memory lost
DECONV          .BS     2               Used to convert decimal numbers
DP_COUNT        .BS     1               Used to determine DP position

READ_COUNTER    .BS     1               Measurement timer
TEMP_IN         .BS     4               Used to read the segments in
HOLD_READINGS   .BS     1               Hold readings if <> 0
CLEAR_STAT      .BS     1               Clear statistics if <> 0

READ_FLAG       .BS     1               Changes every reading

; Next range of values must not be altered
CURRENT_VALUE   .BS     2               Packed current value
;                                       format: b10..b0 is binary value
;                                               b11 is sign flag
;                                               b13 and b12 are dp flags
LAST_DP         .BS     1               Last DP position
MIN_SIGN        .BS     2               Signed minimum value
MIN_ABS         .BS     2               Absolute minimum value
MAX_SIGN        .BS     2               Signed maximum value
MAX_ABS         .BS     2               Absolute maximum value
MEAN_SIGN       .BS     5               Signed mean value
MEAN_ABS        .BS     5               Absolute mean value
MEAN_COUNT      .BS     3               Mean counter
HOLD_VALUE      .BS     2               Value in hold mode
HOLD_DP         .BS     1               DP position for hold values
HOLD_MIN_SIGN   .BS     2               Signed minumum hold value
HOLD_MIN_ABS    .BS     2               Absolute minimum hold value
HOLD_MAX_SIGN   .BS     2               Signed maximum hold value
HOLD_MAX_ABS    .BS     2               Absolute maximum hold value
HOLD_MEAN_SIGN  .BS     5               Signed hold sum for mean
HOLD_MEAN_ABS   .BS     5               Absolute hold sum for mean
HOLD_MEAN_COUNT .BS     3               Hold mean counter

DIV_A           .BS     5               Used for division
DIV_B           .BS     5               Used for division

PARAMETERS      .BS     4               Parameter table

INTERVAL_TIME   .BS     2               Log interval time in seconds
LOG_ACTIVE      .BS     1               Log is active if > 0
;                                       1=Sample, 2=Abs, 3=Signed
LOG_PNTR        .BS     2               Log write pointer
LOG_TIMER       .BS     3               Log interval timer
LOG_MEAN_SUM    .BS     4               Summing for mean log values
LOG_MEAN_COUNT  .BS     4               Counter for mean log values
LOG_DP          .BS     1               DP position during log mode
                .ED                     May not exceed $00DF !

;---------------------------------------------------------------------------
; Declaration of CCU3000 I/O addresses
;---------------------------------------------------------------------------

CPU_SPEED       .EQ     $0200           CPU clock division ration (0=fast)
CPU_CONTROL     .EQ     $0201           CPU control register (source $FFF9)
WATCH_DOG       .EQ     $0202           Watch dog restart register
P1              .EQ     $0203           Port 1 data register
P1_DIR          .EQ     $0204           Port 1 data direction register
P2              .EQ     $0205           Port 2 data register
P2_DIR          .EQ     $0206           Port 2 data direction register
P3              .EQ     $0207           Port 3 data register
P3_DIR          .EQ     $0208           Port 3 data direction register
P4              .EQ     $0209           Port 4 (R/W line)
P5_MODE         .EQ     $020A           Port 5 mode register
P5_DIR          .EQ     $020B           Port 5 data direction register
P5              .EQ     $020C           Port 5 data register
IR_INPUT        .EQ     $020D           IR input flip-flops

IM1_CONTROL     .EQ     $0210           IM-Bus 1 control and status
IM1_RATE        .EQ     $0211           IM-Bus 1 data transfer rate
IM1_ADDRESS     .EQ     $0213           IM-Bus 1 master address
IM1_DATA        .EQ     $0214,$0215     IM-Bus 1 data low,high
IM1_SLAVE1      .EQ     $0216           IM-Bus 1 slave data on IM addr 02
IM1_SLAVE2      .EQ     $0218           IM-Bus 1 slave data on IM addr 03
IM1_SLAVE3      .EQ     $021A           IM-Bus 1 slave data on IM addr 04

INT_CONTROL     .EQ     $021C           Interrupt controller control byte
INT_RETURN      .EQ     $021D           Interrupt controller return byte
INT_SOURCE01    .EQ     $021E           Interrupt priority for sources 0 & 1
INT_SOURCE23    .EQ     $021F           Interrupt priority for sources 2 & 3
INT_SOURCE45    .EQ     $0220           Interrupt priority for sources 4 & 5
INT_SOURCE67    .EQ     $0221           Interrupt priority for sources 6 & 7

T1_CONTROL1     .EQ     $0222           Timer 1 control byte 1
T1_CONTROL2     .EQ     $0223           Timer 1 control byte 2
T1_CONTROL3     .EQ     $0224           Timer 1 control byte 3
T1_PRESC        .EQ     $0225,$0226     Timer 1 prescaler low,high
T1_ACCU         .EQ     $0228,$0229     Timer 1 accu low,high
T1_ADDER        .EQ     $022A,$022B     Timer 1 adder low,high

T2_CONTROL1     .EQ     $022C           Timer 2 control byte 1
T2_CONTROL2     .EQ     $022D           Timer 2 control byte 2
T2_CONTROL3     .EQ     $022E           Timer 2 control byte 3
T2_PRESC        .EQ     $022F,$0230     Timer 2 prescaler low,high
T2_ACCU         .EQ     $0232,$0233     Timer 2 accu low,high
T2_ADDER        .EQ     $0234,$0235     Timer 2 adder low,high

T3_CONTROL1     .EQ     $0236           Timer 3 control byte 1
T3_CONTROL2     .EQ     $0237           Timer 3 control byte 2
T3_CONTROL3     .EQ     $0238           Timer 3 control byte 3
T3_PRESC        .EQ     $0239,$023A     Timer 3 prescaler low,high
T3_ACCU         .EQ     $023C,$023D     Timer 3 accu low,high
T3_ADDER        .EQ     $023E,$023F     Timer 3 adder low,high

P6              .EQ     $0240           Port 6 data register
P6_DIR          .EQ     $0241           Port 6 data direction register
P7              .EQ     $0242           Port 7 data register
P7_DIR          .EQ     $0243           Port 7 data direction register
P8              .EQ     $0244           Port 8 data register
P8_DIR          .EQ     $0245           Port 8 data direction register

IM2_CONTROL     .EQ     $0246           IM-Bus 2 control and status
IM2_RATE        .EQ     $0247           IM-Bus 2 data transfer rate
IM2_ADDRESS     .EQ     $0249           IM-Bus 2 master address
IM2_DATA        .EQ     $024A,$024B     IM-Bus 2 data low,high
IM2_SLAVE1      .EQ     $024C           IM-Bus 2 slave data on IM addr 02
IM2_SLAVE2      .EQ     $024E           IM-Bus 2 slave data on IM addr 03
IM2_SLAVE3      .EQ     $0250           IM-Bus 2 slave data on IM addr 04

;---------------------------------------------------------------------------
; CONSTANTS
;---------------------------------------------------------------------------

; ASCII
BS              .EQ     $08             Back space
LF              .EQ     $0A             Line feed
CR              .EQ     $0D             Carriage return
XON             .EQ     $11             Xon
XOFF            .EQ     $13             Xoff
CANCEL          .EQ     $18             Cancel last transmission
ESC             .EQ     $1B             Escape
DEL             .EQ     $7F             Delete

; EEPROM adresses
EEPROM_WRITE    .EQ     $A0             IC EEPROM write address
EEPROM_READ     .EQ     $A1             IC EEPROM read address

; Memory areas
STACK_BEGIN     .EQ     $01FF           Stack growing down to $01C0 (64bytes)
MAIN_LOG_BUFFER .EQ     $0300,$063B     138x6 bytes for 138x4 readings
SUB_LOG_BUFFER  .EQ     $00E0,$01BE     37x6 bytes for 37x4 readings
FIRST_VALUE     .EQ     $063C,$063D     1st log value (value 0)
WATCH_DOG_SAVE  .EQ     $063E           Last value written to watchdog
DOG_BARKED      .EQ     $063F           Number of times the watchdog barked

; Watchdog
WATCH_DOG_TIME  .EQ     8               Watchdog delay about 0.5 sec @ 1MHz

; Interrupt constants
T1_PRIORITY     .EQ     %0000.0101      T1=SB-bus input, highest priority
T2_PRIORITY     .EQ     %0010.0000      T2=SB-bus output, lowest priority
T3_PRIORITY     .EQ     %0000.0011      T3=5ms timer, middle priority
P87_PRIORITY    .EQ     %0100.0000      P87 2nd highest priority

; Timer constants
SIN_MODE1       .EQ     %0010.0111      T_CONTROL1 value for serial input
SIN_MODE2       .EQ     %0000.1010      T_CONTROL2 value for serial input
SIN_MODE3       .EQ     %0110.1111      T_CONTROL3 value for serial input
SOUT_MODE1      .EQ     %1011.0000      T_CONTROL1 value for serial output
SOUT_MODE2      .EQ     %0100.0010      T_CONTROL2 value for serial output
SOUT_MODE3      .EQ     %0110.1111      T_CONTROL3 value for serial output
TIME_MODE1      .EQ     %1010.0000      T_CONTROL1 value for interval timer
TIME_MODE2      .EQ     %0000.0001      T_CONTROL2 value for interval timer
TIME_MODE3      .EQ     %1001.0110      T_CONTROL3 value for interval timer
PRESC_9600      .EQ     416             Prescaler for 9600 baud (Fosc/9600)
TIMEOUT_5MS     .EQ     20000           Timeout value for 5ms intervals

; General purpose constants
MAX_ACK_ERRORS  .EQ     10              Maximum acknowledge errors

;---------------------------------------------------------------------------
; MACRO    :  SAY
; FUNCTION :  Send a string to the SB-Bus
;---------------------------------------------------------------------------

SAY             .MA     STRINGPOINTER
                LDX     #]1             Get low byte of string pointer
                LDY     /]1              and high byte of string pointer
                JSR     SEND_STRING     Send the string to SB-Bus
                .EM

