;---------------------------------------------------------------------------
;
; meter.asm
;
; Include file for Fluke SB-Bus interface
;
; Assembling started at 25-4-1997
;
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
; ROUTINE    :  READ_SEGMENTS
; FUNCTION   :  Read all segments and put them in TEMP_IN (4 bytes)
;---------------------------------------------------------------------------

READ_SEGMENTS   LDA     P5              Get all segments in
                AND     #%0011.1111
                STA     TEMP_IN
                LDA     P6
                AND     #%0111.1111
                STA     TEMP_IN+1
                LDA     P7
                AND     #%0011.1111
                STA     TEMP_IN+2
                LDA     P8
                AND     #%1000.0000
                STA     TEMP_IN+3

                LDA     P5              Compare them with last reading
                AND     #%0011.1111      in case display just changed
                CMP     TEMP_IN
                BNE     READ_SEGMENTS
                LDA     P6
                AND     #%0111.1111
                CMP     TEMP_IN+1
                BNE     READ_SEGMENTS
                LDA     P7
                AND     #%0011.1111
                CMP     TEMP_IN+2
                BNE     READ_SEGMENTS
                LDA     P8
                AND     #%1000.0000
                CMP     TEMP_IN+3
                BNE     READ_SEGMENTS
                RTS

;---------------------------------------------------------------------------
; ROUTINE    :  GET_DP
; FUNCTION   :  Get DP position and correct phase
;---------------------------------------------------------------------------

GET_DP          LDA     TEMP_IN+3       Combine DPs to a 3 bit code
                ORA     TEMP_IN+1
                LSR
                LSR
                LSR
                LSR
                LSR
                AND     #%0000.0111
                TAX                     Convert this code to 2 bits
                LDA     .DP_TABLE,X
                STA     TEMP_IN+3       Save DP position
                BPL     .PHASE_OK       Phase is OK!
                AND     #%0000.0011     Throw away phase flag to keep
                STA     TEMP_IN+3        the propper DP position
                LDA     TEMP_IN         Reverse all phases
                EOR     #$FF
                STA     TEMP_IN
                LDA     TEMP_IN+1
                EOR     #$FF
                STA     TEMP_IN+1
                LDA     TEMP_IN+2
                EOR     #$FF
                STA     TEMP_IN+2
.PHASE_OK       RTS

.DP_TABLE       .DA     #%0000.0000     000 > No DPs are on, OK
                .DA     #%0000.0001     001 > DP1 is on, OK
                .DA     #%0000.0010     010 > DP2 is on, OK
                .DA     #%1000.0011     011 > DP1 and DP2 are on, not OK
                .DA     #%0000.0011     100 > DP3 is on, OK
                .DA     #%1000.0010     101 > DP1 and DP3 are on, not OK
                .DA     #%1000.0001     110 > DP2 and DP3 are on, not OK
                .DA     #%1000.0000     111 > All DPs are on, not OK

;---------------------------------------------------------------------------
; ROUTINE    :  SEGMENT_2_DEC
; FUNCTION   :  Convert segment pattern to decimal
;---------------------------------------------------------------------------

SEGMENT_2_DEC   STZ     CURRENT_VALUE   Clear current value
                STZ     CURRENT_VALUE+1
                LDA     TEMP_IN         Get OR1 segment
                AND     #%0010.0000
                BEQ     .NOT_OR1        OR1 is not on!
                LDA     #10             Store 10 (1 x10 = 10) to current
                STA     CURRENT_VALUE    value

.NOT_OR1        LDA     TEMP_IN+2       Convert MSD's segments to decimal
                JSR     .CONVERT
                BMI     .OL             Overload situation!
                ADD     CURRENT_VALUE   Add this to the current value
                STA     CURRENT_VALUE
                JSR     .MUL_10         Multiply current value with 10

                LDA     TEMP_IN+1       Convert middle digit to decimal
                JSR     .CONVERT
                ADD     CURRENT_VALUE   Add this to the current value again
                STA     CURRENT_VALUE
                JSR     .MUL_10          and multiply by 10 again

                LDA     TEMP_IN         Convert LSD to decimal
                JSR     .CONVERT
                ADD     CURRENT_VALUE   This gives the current value!
                STA     CURRENT_VALUE
                STA     TEMP_IN
                LDA     CURRENT_VALUE+1
                ADC     #0
                STA     CURRENT_VALUE+1
                STA     TEMP_IN+1
                BRA     .NOT_OL

.OL             LDA     #$FF            Indicate OL situation by setting
                STA     CURRENT_VALUE    11 bits to 1
                STA     TEMP_IN
                LDA     #%0000.0111
                STA     CURRENT_VALUE+1
                STA     TEMP_IN+1
.NOT_OL         RTS

.CONVERT        AND     #%0001.1111     Only first 5 bits are important
                LDX     #9              Setup counter and table pointer
.CONVERT_LOOP   CMP     .SEGMENT_TABLE,X  Compare segments
                BEQ     .CONVERT_DONE   Found matching pattern
                DEX
                BPL     .CONVERT_LOOP   Do this for all normal digits
.CONVERT_DONE   TXA                     Get decimal value of digit in A
                RTS

.MUL_10         ASL     CURRENT_VALUE   Multiply current value by 10
                ROL     CURRENT_VALUE+1
                LDA     CURRENT_VALUE
                LDX     CURRENT_VALUE+1
                ASL     CURRENT_VALUE
                ROL     CURRENT_VALUE+1
                ASL     CURRENT_VALUE
                ROL     CURRENT_VALUE+1
                ADD     CURRENT_VALUE
                STA     CURRENT_VALUE
                TXA
                ADC     CURRENT_VALUE+1
                STA     CURRENT_VALUE+1
                RTS

.SEGMENT_TABLE  .DA     #%01111         0   0
                .DA     #%00010         1   1
                .DA     #%10111         2   3
                .DA     #%10011         3   3
                .DA     #%11010         4   *
                .DA     #%11001         5   *
                .DA     #%11101         6   6
                .DA     #%00011         7   7
                .DA     #%11111         8   8
                .DA     #%11011         9   3

;---------------------------------------------------------------------------
; ROUTINE    :  POL_DP_2_CV
; FUNCTION   :  Add POL and DP to current value
;---------------------------------------------------------------------------

POL_DP_2_CV     LDA     TEMP_IN+2       Get polarity flag
                AND     #%0010.0000
                BEQ     .POSITIVE       Value is positive!
                LDA     CURRENT_VALUE+1 Set negative sign
                ORA     #%0000.1000
                STA     CURRENT_VALUE+1
.POSITIVE
                LDA     TEMP_IN+3       Save DP position too
                ASL
                ASL
                ASL
                ASL
                ORA     CURRENT_VALUE+1
                STA     CURRENT_VALUE+1 Current value is now complete
                RTS

;---------------------------------------------------------------------------
; ROUTINE    :  PREPARE4STATIST
; FUNCTION   :  Prepare values for statistical functions
;---------------------------------------------------------------------------

PREPARE4STATIST LDX     TEMP_IN+3       Save current DP position
                LDA     TEMP_IN         Copy value for signed calculation
                STA     TEMP_IN+2
                LDA     TEMP_IN+1
                STA     TEMP_IN+3
                EOR     #%0000.0111     Overload situation?
                BNE     .IN_RANGE       No!
                LDA     TEMP_IN
                INC
                BNE     .IN_RANGE       No!
                LDA     #9999           Use a value of 9999 in case of OL
                STA     TEMP_IN
                STA     TEMP_IN+2
                LDA     /9999
                STA     TEMP_IN+1
                STA     TEMP_IN+3
.IN_RANGE
                LDA     CURRENT_VALUE+1 Is the number negative?
                AND     #%0000.1000
                BEQ     .NOT_NEGATIVE   No!
                LDA     TEMP_IN+2       Calculate 2s complement for signed
                EOR     #$FF             calculations
                ADD     #1
                STA     TEMP_IN+2
                LDA     TEMP_IN+3
                EOR     #$FF
                ADC     #0
                STA     TEMP_IN+3
.NOT_NEGATIVE   RTS

;---------------------------------------------------------------------------
; ROUTINE  :  CLR_STATIST
; FUNCTION :  Clear statistic values if requested
;---------------------------------------------------------------------------

CLR_STATIST     CPX     LAST_DP         See if DP still at same position
                BEQ     .CLEAR          Yes! Test normal clear flag
                STX     LAST_DP         Save new DP position
                LDA     #-1             Avoid false readings when switching
                STA     CLEAR_STAT       range by clearing only next time!
                STZ     LOG_ACTIVE      Clear log active mode
                RTS
.CLEAR          LDA     CLEAR_STAT      Clear statistics requested?
                BEQ     .RTS            No!
                STZ     CLEAR_STAT      Clear clear-flag
                STZ     MAX_ABS         Minimum value here is 0
                STZ     MAX_ABS+1
                LDA     #-9999          Minimum value here is -9999
                STA     MAX_SIGN
                LDA     /-9999
                STA     MAX_SIGN+1
                LDA     #9999           Maximum value here is 9999 in both
                STA     MIN_SIGN         cases
                STA     MIN_ABS
                LDA     /9999
                STA     MIN_SIGN+1
                STA     MIN_ABS+1
                LDX     #12             Clear a total of 13 mean bytes
.CLR_MEAN       STZ     MEAN_SIGN,X      (2x5 summing + 3 counter)
                DEX
                BPL     .CLR_MEAN
.RTS            RTS

;---------------------------------------------------------------------------
; ROUTINE  :  SUM_MEAN
; FUNCTION :  Sum values for mean function
;---------------------------------------------------------------------------

; Add value to both mean summing registers
SUM_MEAN        LDA     MEAN_COUNT      Increment number of samples
                ADD     #1
                STA     MEAN_COUNT
                LDA     MEAN_COUNT+1
                ADC     #0
                STA     MEAN_COUNT+1
                LDA     MEAN_COUNT+2
                ADC     #0
                STA     MEAN_COUNT+2
                LDA     TEMP_IN         Absolute value
                ADD     MEAN_ABS
                STA     MEAN_ABS
                LDA     TEMP_IN+1
                ADC     MEAN_ABS+1
                STA     MEAN_ABS+1
                LDA     MEAN_ABS+2
                ADC     #0
                STA     MEAN_ABS+2
                LDA     MEAN_ABS+3
                ADC     #0
                STA     MEAN_ABS+3
                LDA     MEAN_ABS+4
                ADC     #0
                STA     MEAN_ABS+4

                LDX     #0              Sign extend in case of postive number
                LDA     TEMP_IN+2       Signed value
                ADD     MEAN_SIGN
                STA     MEAN_SIGN
                LDA     TEMP_IN+3
                BPL     .POS            It's a positive number
                DEX                     Make sign extend -1
.POS            ADC     MEAN_SIGN+1
                STA     MEAN_SIGN+1
                TXA
                ADC     MEAN_SIGN+2
                STA     MEAN_SIGN+2
                TXA
                ADC     MEAN_SIGN+3
                STA     MEAN_SIGN+3
                TXA
                ADC     MEAN_SIGN+4
                STA     MEAN_SIGN+4
                RTS

;---------------------------------------------------------------------------
; ROUTINE  :  DO_MIN_MAX
; FUNCTION :  Handle minimum and maximum values
;---------------------------------------------------------------------------

; Handle absolute minimum
DO_MIN_MAX      LDA     TEMP_IN         See if unsigned value < minimum
                CMP     MIN_ABS
                LDA     TEMP_IN+1
                SBC     MIN_ABS+1
                BCS     .SKIP1          Current>minimum, don't store new!
                LDA     TEMP_IN         Save new absolute minimum
                STA     MIN_ABS
                LDA     TEMP_IN+1
                STA     MIN_ABS+1

; Handle absolute maximum
.SKIP1          LDA     MAX_ABS         See if unsigned value > maximum
                CMP     TEMP_IN
                LDA     MAX_ABS+1
                SBC     TEMP_IN+1
                BCS     .SKIP2          Current<maximum, don't store new!
                LDA     TEMP_IN         Save new absolute maximum
                STA     MAX_ABS
                LDA     TEMP_IN+1
                STA     MAX_ABS+1

; Handle signed minimum
.SKIP2          LDA     TEMP_IN+3       Are both signs equal?
                EOR     MIN_SIGN+1
                BMI     .MIN_DIFFER     No! Handle differently
                LDA     TEMP_IN+2       See if signed value < minimum
                CMP     MIN_SIGN
                LDA     TEMP_IN+3
                SBC     MIN_SIGN+1
                BCS     .SKIP3          Current>minimum, don't store new!
.COPY_MIN_SIGN  LDA     TEMP_IN+2       Save new signed minimum
                STA     MIN_SIGN
                LDA     TEMP_IN+3
                STA     MIN_SIGN+1
                BRA     .SKIP3
.MIN_DIFFER     LDA     TEMP_IN+3       Is the new value negative?
                BMI     .COPY_MIN_SIGN  Yes! Then that's less!

; Handle signed maximum
.SKIP3          LDA     TEMP_IN+3       Are both signs equal?
                EOR     MAX_SIGN+1
                BMI     .MAX_DIFFER     No! Handle differently
                LDA     MAX_SIGN        See if signed value > maximum
                CMP     TEMP_IN+2
                LDA     MAX_SIGN+1
                SBC     TEMP_IN+3
                BCS     .SKIP4          Current<maximum, don't store new!
.COPY_MAX_SIGN  LDA     TEMP_IN+2       Save new signed maximum
                STA     MAX_SIGN
                LDA     TEMP_IN+3
                STA     MAX_SIGN+1
                BRA     .SKIP4
.MAX_DIFFER     LDA     TEMP_IN+3       Is the new value positive?
                BPL     .COPY_MAX_SIGN  Yes! Then that's more!

.SKIP4          RTS

;---------------------------------------------------------------------------
; ROUTINE  :  CHECK_HOLD
; FUNCTION :  See if readings must be copied to hold values
;             Value can't change in between as it is part of IRQ routine
;---------------------------------------------------------------------------

CHECK_HOLD      LDA     HOLD_READINGS   Is the flag set?
                BEQ     .RTS            No! Don't do anything
                STZ     HOLD_READINGS   Clear the flag
                LDX     #23             Copy 24 bytes
.LOOP           LDA     CURRENT_VALUE,X
                STA     HOLD_VALUE,X
                DEX
                BPL     .LOOP           Do them all!
.RTS            RTS

;---------------------------------------------------------------------------
; ROUTINE  :  SUM_LOG_MEAN
; FUNCTION :  Sum this reading to the log mean registers
;---------------------------------------------------------------------------

SUM_LOG_MEAN    LDA     LOG_ACTIVE      Is the log function active?
                BEQ     .RTS            No! Don't bother
                DEC
                BEQ     .RTS            Yes! But in momentary value mode
                DEC
                BNE     .SUM_SIGNED     Sum in signed value mode
                LDA     TEMP_IN         Add current absolute value to the
                ADD     LOG_MEAN_SUM     summing register
                STA     LOG_MEAN_SUM
                LDA     TEMP_IN+1
                ADC     LOG_MEAN_SUM+1
                STA     LOG_MEAN_SUM+1
                LDX     #0              Add 0 to the highest bytes (with C)
                BRA     .SKIP
.SUM_SIGNED     LDA     TEMP_IN+2       Add current signed value to the
                ADD     LOG_MEAN_SUM     summing register
                STA     LOG_MEAN_SUM
                LDX     #0              In case it's a positive value
                LDA     TEMP_IN+3
                BPL     .POSITIVE       It's a positive number
                LDX     #-1             Used for sign extend
.POSITIVE       ADC     LOG_MEAN_SUM+1
                STA     LOG_MEAN_SUM+1
.SKIP           TXA                     Add carry to the 2 highest bytes
                ADC     LOG_MEAN_SUM+2
                STA     LOG_MEAN_SUM+2
                TXA
                ADC     LOG_MEAN_SUM+3
                STA     LOG_MEAN_SUM+3
                INC     LOG_MEAN_COUNT  Increment summing counter
                BNE     .RTS            No carry!
                INC     LOG_MEAN_COUNT+1
                BNE     .RTS            No carry!
                INC     LOG_MEAN_COUNT+2
                BNE     .RTS            No carry!
                INC     LOG_MEAN_COUNT+3
.RTS            RTS

;---------------------------------------------------------------------------
; ROUTINE  :  LOG_SAMPLE
; FUNCTION :  See if a sample must be stored to the log
;---------------------------------------------------------------------------

LOG_SAMPLE      LDA     LOG_ACTIVE      Is the log function active?
                BNE     .DO_IT          Yes! Do it
                LDA     P8              Switch off the BT indicator
                ORA     #%0000.0001
                STA     P8
                RTS
.DO_IT          LDA     READ_COUNTER    Flash BT indicator every reading
                CMP     #20
                BNE     .NO_FLASH
                LDA     P8              Flash the indicator
                EOR     #%0000.0001
                STA     P8
.NO_FLASH       LDA     LOG_TIMER       Decrement log timer
                SUB     #1
                STA     LOG_TIMER
                LDA     LOG_TIMER+1
                SBC     #0
                STA     LOG_TIMER+1
                LDA     LOG_TIMER+2
                SBC     #0
                STA     LOG_TIMER+2
                ORA     LOG_TIMER+1
                ORA     LOG_TIMER
                BNE     .RTS            Not timed out yet!
.DO_LOG         LDA     LOG_PNTR        Is it the 1st value to be stored?
                ORA     LOG_PNTR+1
                BNE     .NOT_1ST        Not the 1st!
.LOG_MOMENT     LDX     CURRENT_VALUE+1 Get default signed high byte
                LDA     LOG_ACTIVE      Was it absolute mode?
                CMP     #2
                BNE     .NOT_ABSOLUTE   No!
                TXA                     Make value absolute
                AND     #%1111.0111
                TAX
.NOT_ABSOLUTE   LDA     CURRENT_VALUE   Save this value to the log file
                JSR     .STORE
                LDX     #7              Clear sum and counter registers
.CLR_STAT1      STZ     LOG_MEAN_SUM,X
                DEX
                BPL     .CLR_STAT1      Do all 8 bytes
                BRA     .RE_INIT        Re-initialize for next reading
.NOT_1ST        LDA     LOG_ACTIVE      What kind of logging?
                DEC
                BEQ     .LOG_MOMENT     Momentary value!
                LDA     INTERVAL_TIME   Interval 0 (store every reading)?
                ORA     INTERVAL_TIME+1
                BEQ     .LOG_MOMENT     No need to calculate mean value!
                JSR     .CALC_MEAN      Caculate mean value
                JSR     .STORE          Store this value
.RE_INIT        INC     LOG_PNTR        Increment log pointer
                BNE     .NO_CARRY       No carry to high byte!
                INC     LOG_PNTR+1
.NO_CARRY       JSR     .CALC_TIME      Calculate interval time

                LDA     LOG_PNTR        End of log file reached?
                CMP     #701
                BNE     .RTS            No!
                LDA     LOG_PNTR+1
                CMP     /701
                BNE     .RTS            No!
                STZ     LOG_ACTIVE      Switch off the log mode
                LDA     P8              Switch off the BT indicator
                ORA     #%0000.0001
                STA     P8
.RTS            RTS

.STORE          PHX                     Save value to be stored
                PHA
                LDA     LOG_PNTR        Store first value of the log?
                ORA     LOG_PNTR+1
                BNE     .NOT_LOG_0      No!
                PLA                     Save first value in special
                STA     FIRST_VALUE      location
                PLA
                STA     FIRST_VALUE+1
                RTS

.NOT_LOG_0      LDA     LOG_PNTR+1      Copy current log pointer and
                STA     TEMP_IN+3        decrement it
                LDA     LOG_PNTR
                BNE     .NO_BORROW
                DEC     TEMP_IN+3
.NO_BORROW      DEC
                STA     TEMP_IN+2

                LDA     TEMP_IN+3       Calculate group address
                LSR                      If ptr<552 : ptr/4*6+$0300
                STA     TEMP_IN+1        If ptr>551 : (ptr-552)/4*6+$00E0
                LDA     TEMP_IN+2
                ROR
                AND     #%1111.1110     /4*2
                STA     TEMP_IN
                LDA     TEMP_IN+2
                AND     #%1111.1100     /4*4
                ADD     TEMP_IN
                STA     TEMP_IN
                LDA     TEMP_IN+3
                ADC     TEMP_IN+1
                STA     TEMP_IN+1
                LDA     TEMP_IN         See if group address >= 138*6
                CMP     #138*6
                LDA     TEMP_IN+1
                SBC     /138*6
                BCS     .2ND            Use 2nd buffer area
                LDA     TEMP_IN         Add begin address of 1st buffer to
                ADD     #MAIN_LOG_BUFFER  the current pointer
                STA     TEMP_IN
                LDA     TEMP_IN+1
                ADC     /MAIN_LOG_BUFFER
                STA     TEMP_IN+1
                BRA     .1ST
.2ND            LDA     TEMP_IN         Add begin address of 1st buffer
                ADD     #-138*6+SUB_LOG_BUFFER  minus 138*6 to current pointer
                STA     TEMP_IN
                LDA     TEMP_IN+1
                ADC     /-138*6+SUB_LOG_BUFFER
                STA     TEMP_IN+1

.1ST            LDA     TEMP_IN+2       Get index in this group
                AND     #%0000.0011     (4 12bit readings in each group)
                CMP     #%0000.0011     Is it the specially packed reading?
                BEQ     .SPECIAL        Yes!
                ASL                     Use this pointer as index
                TAY
                PLA                     Store the value
                STA     (TEMP_IN),Y
                INY
                PLA
                AND     #%0000.1111
                STA     (TEMP_IN),Y
                RTS

.SPECIAL        PLA                     Get low nibble of low byte
                PHA
                ASL
                ASL
                ASL
                ASL
                LDY     #1
                ORA     (TEMP_IN),Y     Save LS nibble
                STA     (TEMP_IN),Y
                INY
                INY
                PLA                     Get high nibble of low byte
                AND     #%1111.0000
                ORA     (TEMP_IN),Y     Save middle nibble
                STA     (TEMP_IN),Y
                INY
                INY
                PLA                     Get low nibble of high byte
                ASL
                ASL
                ASL
                ASL
                ORA     (TEMP_IN),Y     Save MS nibble
                STA     (TEMP_IN),Y
                RTS

.CALC_MEAN      LDA     LOG_MEAN_SUM+3  Save sign of the sum
                PHA
                BPL     .POSITIVE       It's a positive value
                LDA     LOG_MEAN_SUM    Calculate 2s complement
                EOR     #$FF
                ADD     #1
                STA     LOG_MEAN_SUM
                LDA     LOG_MEAN_SUM+1
                EOR     #$FF
                ADC     #0
                STA     LOG_MEAN_SUM+1
                LDA     LOG_MEAN_SUM+2
                EOR     #$FF
                ADC     #0
                STA     LOG_MEAN_SUM+2
                LDA     LOG_MEAN_SUM+3
                EOR     #$FF
                ADC     #0
                STA     LOG_MEAN_SUM+3

.POSITIVE       STZ     TEMP_IN         Clear result register
                STZ     TEMP_IN+1

                LDA     LOG_MEAN_SUM    Is sum 0?
                ORA     LOG_MEAN_SUM+1
                ORA     LOG_MEAN_SUM+2
                ORA     LOG_MEAN_SUM+3
                BNE     .JUSTIFY        No!
                PLA                     Clear stack
                LDA     #0              Make the returned value 0
                TAX
                RTS

.JUSTIFY        LDA     LOG_MEAN_SUM+3  Left justify sum
                BMI     .JUSTIFIED      Done!
                ASL     LOG_MEAN_SUM    Shift sum 1 position left
                ROL     LOG_MEAN_SUM+1
                ROL     LOG_MEAN_SUM+2
                ROL     LOG_MEAN_SUM+3
                ASL     LOG_MEAN_COUNT  Do the same with divider
                ROL     LOG_MEAN_COUNT+1
                ROL     LOG_MEAN_COUNT+2
                ROL     LOG_MEAN_COUNT+3
                BRA     .JUSTIFY        Try again until justified

.JUSTIFIED      LDY     #0              Shift counter left until justified
.SHIFT_COUNT    LDA     LOG_MEAN_COUNT+3
                BMI     .DIVIDE_LOOP    Done if left bit is 1!
                INY                     Count number of shifts
                ASL     LOG_MEAN_COUNT
                ROL     LOG_MEAN_COUNT+1
                ROL     LOG_MEAN_COUNT+2
                ROL     LOG_MEAN_COUNT+3
                BRA     .SHIFT_COUNT

.DIVIDE_LOOP    LDA     LOG_MEAN_SUM    See if counter goes into sum
                CMP     LOG_MEAN_COUNT
                LDA     LOG_MEAN_SUM+1
                SBC     LOG_MEAN_COUNT+1
                LDA     LOG_MEAN_SUM+2
                SBC     LOG_MEAN_COUNT+2
                LDA     LOG_MEAN_SUM+3
                SBC     LOG_MEAN_COUNT+3
                BCC     .NO_GO          It didn't go!
                LDA     LOG_MEAN_SUM    Realy subtract it now
                SUB     LOG_MEAN_COUNT
                STA     LOG_MEAN_SUM
                LDA     LOG_MEAN_SUM+1
                SBC     LOG_MEAN_COUNT+1
                STA     LOG_MEAN_SUM+1
                LDA     LOG_MEAN_SUM+2
                SBC     LOG_MEAN_COUNT+2
                STA     LOG_MEAN_SUM+2
                LDA     LOG_MEAN_SUM+3
                SBC     LOG_MEAN_COUNT+3
                STA     LOG_MEAN_SUM+3
.NO_GO          ROL     TEMP_IN         Roll C into result
                ROL     TEMP_IN+1
                LSR     LOG_MEAN_COUNT+3 Shift divider to the right
                ROR     LOG_MEAN_COUNT+2
                ROR     LOG_MEAN_COUNT+1
                ROR     LOG_MEAN_COUNT
                DEY                     Decrement itteration counter
                BPL     .DIVIDE_LOOP    Do it again!

                LDA     TEMP_IN         See if value is overload
                CMP     #2000
                LDA     TEMP_IN+1
                SBC     /2000
                BCC     .NOT_OL         It's not!
                LDA     #%1111.1111     Set OL situation
                STA     TEMP_IN
                LDA     #%0000.0111
                STA     TEMP_IN+1

.NOT_OL         PLA                     Get sign
                BPL     .IS_POS         Value was positive!
                LDA     TEMP_IN+1       Set negative sign
                ORA     #%0000.1000
                STA     TEMP_IN+1
.IS_POS         LDA     LOG_DP          Combine with DP position
                ASL
                ASL
                ASL
                ASL
                ORA     TEMP_IN+1
                STA     TEMP_IN+1
                LDX     #7              Clear sum and counter registers
.CLR_STAT2      STZ     LOG_MEAN_SUM,X
                DEX
                BPL     .CLR_STAT2      Do all 8 bytes
                LDA     TEMP_IN         Get the return value
                LDX     TEMP_IN+1
                RTS

.CALC_TIME      LDA     INTERVAL_TIME   Interval 0 (log every reading)?
                ORA     INTERVAL_TIME+1
                BNE     .SECONDS        No! Calculate timeout value
                LDA     #20             Set the timer to 20 interrupts
                STA     LOG_TIMER
                STZ     LOG_TIMER+1
                STZ     LOG_TIMER+2
                RTS
.SECONDS        LDA     INTERVAL_TIME+1 Multiply interval time with 50
                STA     TEMP_IN+1
                LDA     INTERVAL_TIME
                ASL
                ROL     TEMP_IN+1
                STA     TEMP_IN
                ADD     INTERVAL_TIME
                STA     LOG_TIMER
                LDA     TEMP_IN+1
                ADC     INTERVAL_TIME+1
                STA     LOG_TIMER+1
                STZ     LOG_TIMER+2
                ASL     LOG_TIMER
                ROL     LOG_TIMER+1
                ROL     LOG_TIMER+2
                ASL     LOG_TIMER
                ROL     LOG_TIMER+1
                ROL     LOG_TIMER+2
                ASL     LOG_TIMER
                ROL     LOG_TIMER+1
                ROL     LOG_TIMER+2
                ASL     LOG_TIMER
                ROL     LOG_TIMER+1
                ROL     LOG_TIMER+2
                LDA     TEMP_IN
                ADD     LOG_TIMER
                STA     LOG_TIMER
                LDA     TEMP_IN+1
                ADC     LOG_TIMER+1
                STA     LOG_TIMER+1
                LDA     LOG_TIMER+2
                ADC     #0
                STA     LOG_TIMER+2
                RTS

