;---------------------------------------------------------------------------
;
; system.asm
;
; Include file for Fluke SB-Bus interface
;
; Assembling started at 25-4-1997
;
;---------------------------------------------------------------------------

;---------------------------------------------------------------------------
; ROM ID
;---------------------------------------------------------------------------

                .AS     /Fluke 8010, 8012 meter interface/

;---------------------------------------------------------------------------
; ROUTINE    :  SIN_IRQ
; FUNCTION   :  SB-Bus input service routine
;---------------------------------------------------------------------------

SIN_IRQ         CLD                     Always run in binary mode
                PHA                     Save registers during interrupts
                PHX
                PHY

                LDA     T1_ADDER        Get received data asap, to free T1
                BMI     .NEWADDRESS     It's an address byte!
                LDX     SELECTED        Are we selected?
                BEQ     .RTI_STEP       No! Forget the character!
                LDX     S_HALTED        Serial interface halted?
                BEQ     .DATA           No! Normal data
                STZ     S_HALTED        Restart serial interface
                CMP     #ESC            Is it ESC?
                BNE     .RTI_STEP       No! Finished
                STA     STOP_ACT        Set stop activities flag
                JMP     .RTI

.DATA           LDX     SB_IN_HEAD      Setup input buffer pointer
                CMP     #' '            Is it a control character?
                BCC     .CTRL           Yes!
                JSR     STORE_IN        Save character in input buffer
.RTI_STEP       JMP     .RTI            Done!

.CTRL           CMP     #BS             Backspace character?
                BEQ     .BSDEL          Yes!
                CMP     #DEL            Del character?
                BNE     .IS_CR          No!
.BSDEL          LDA     INLENGTH        Input buffer empty?
                BEQ     .RTI_STEP       Yes! Don't backspace anymore
                LDA     SB_IN_HEAD      Decrement input pointer
                DEC
                AND     #%0011.1111     Limit to 64 bytes
                STA     SB_IN_HEAD
                DEC     INLENGTH        Decrement input length counter
                BRA     .RTI_STEP       Done!

.IS_CR          CMP     #CR             Carriage return?
                BNE     .IS_XOF         No!
                JSR     STORE_IN        Store last character
                BRA     .RTI            Done

.IS_XOF         CMP     #XOFF           Stop serial I/O?
                BNE     .IS_CAN         No!
                STA     S_HALTED        Set XOFF flag (<>0)
                BRA     .RTI            Done

.IS_CAN         CMP     #CANCEL         Clear input buffer?
                BNE     .IS_ESC         No!
.CAN_LOOP       LDA     INLENGTH        Backspace back until begin
                BEQ     .RTI            Done!
                LDA     SB_IN_HEAD      Decrement pointer
                DEC
                AND     #%0011.1111     Limit to 64 bytes
                STA     SB_IN_HEAD
                DEC     INLENGTH        Decrement input length counter
                BRA     .CAN_LOOP

.IS_ESC         CMP     #ESC            Escape character?
                BNE     .RTI            No! Done
                STA     STOP_ACT        Make stop activities flag <> 0
                BRA     .RTI            Done!

.NEWADDRESS     TAX                     Save input address
                LDA     #-1             Set special situation
                STA     LAST_SEND
                STA     STOP_ACT        Stop activities
                STZ     SELECTED        Deselect for a start
                STZ     S_HALTED
                STZ     CR_COUNT
                STZ     CR_DELAY
                STZ     INLENGTH
                STZ     SB_IN_HEAD      Clear input and output buffers
                STZ     SB_IN_TAIL
                STZ     SB_OUT_HEAD
                STZ     SB_OUT_TAIL
                LDA     HOLD_ACTIVE     Hold mode active?
                BEQ     .HOLD           No! Don't bother then
                LDA     HOLD_FUNCTION   Hold mode empty?
                ORA     HOLD_FUNCTION+1
                BNE     .HOLD           No! Keep it like this
                STZ     HOLD_ACTIVE     Clear empty hold
.HOLD           CPX     #255            General call address?
                BNE     .NOTALL         No!
                LDA     #%0100.0000     Select as general call
                STA     SELECTED
                BRA     .RTI            Done
.NOTALL         CPX     MY_ADDRESS      Own bus address?
                BNE     .RTI            No! Not selected!
                LDA     #%1000.0000     Select as private call
                STA     SELECTED
                LDA     #'='            Send => to the output buffer
                STA     OUTPUT_BUFFER    (don't use system routines
                LDA     #'>'              for they are not reantrant)
                STA     OUTPUT_BUFFER+1
                LDA     #CR
                STA     OUTPUT_BUFFER+2
                LDA     #3              Setup initial output
                STA     SB_OUT_HEAD      buffer pointer
                STZ     ERROR           Clear last error code

.RTI            PLY                     Restore used registers
                PLX
                PLA

                STA     INT_RETURN
                RTI

;---------------------------------------------------------------------------
; ROUTINE    :  SOUT_IRQ
; FUNCTION   :  SB-Bus output service routine
;---------------------------------------------------------------------------

SOUT_IRQ        CLD                     Always run in binary mode
                PHA                     Save registers during interrupts
                PHX
                PHY

                JSR     SOUT            Send serial character out

                PLY                     Restore registers
                PLX
                PLA
                STA     INT_RETURN
                RTI

;---------------------------------------------------------------------------
; ROUTINE    :  INTERVAL_IRQ
; FUNCTION   :  5ms interval service routine
;---------------------------------------------------------------------------

INTERVAL_IRQ    CLD                     Always run in binary mode
                PHA                     Save used registers
                LDA     CR_DELAY        Is CR delay running?
                BEQ     .RTI            No!
                DEC     CR_DELAY
.RTI            PLA                     Restore used registers
                STA     INT_RETURN
                RTI

;---------------------------------------------------------------------------
; ROUTINE    :  DISPLAY_IRQ
; FUNCTION   :  Triggered by display clock
;---------------------------------------------------------------------------

DISPLAY_IRQ     CLD                     Always run in binary mode
                PHA                     Save used registers
                PHX
                PHY

                DEC     READ_COUNTER    Decrement read counter
                BNE     .DONT_READ      Counter not 0 yet!
                LDA     #20             20 pulses per measurement @ 50Hz
                STA     READ_COUNTER     gives 2 1/2 readings per second
                INC     READ_FLAG       Change the read flag
                JSR     READ_SEGMENTS   Read segment patterns
                JSR     GET_DP          Get DP position and correct phase
                JSR     SEGMENT_2_DEC   Convert segment pattern to decimal
                JSR     POL_DP_2_CV     Add pol and DP to current value
                JSR     PREPARE4STATIST Prepare values for statistics
                JSR     CLR_STATIST     Clear statistics if neccessery
                JSR     SUM_MEAN        Sum values for mean functions
                JSR     DO_MIN_MAX      Handle minimum and maximum values
                JSR     SUM_LOG_MEAN    Sum mean value for log function

.DONT_READ      JSR     CHECK_HOLD      See if readings must be held
                JSR     LOG_SAMPLE      See if a log sample must be taken

.RTI            PLY                     Restore used registers
                PLX
                PLA
                STA     INT_RETURN
                RTI

;---------------------------------------------------------------------------
; ROUTINE    :  RESET
; FUNCTION   :  Get the machine started
;---------------------------------------------------------------------------

RESET           CLD                     Is automatic, but do it anyway
                LDX     #STACK_BEGIN    Initialize stack pointer
                TXS
                LDA     #3              Run CPU at 1MHz to save current
                STA     CPU_SPEED

; Check the watch dog
                LDA     WATCH_DOG       Reset caused by the watch-dog?
                CMP     #%0000.0010
                BEQ     .DOG_BARKED     Yes! Increment bark counter
                STZ     DOG_BARKED      Clear barked counter
                BRA     .WAKE_DOG       Now wake the dog
.DOG_BARKED     INC     DOG_BARKED      Increment dog barked counter
                BNE     .WAKE_DOG       No overflow!
                DEC     DOG_BARKED      Limit to 255 times
.WAKE_DOG       LDA     #WATCH_DOG_TIME Start the watchdog timer
                .DO     DOG_HOME         only if we want it to
                STA     WATCH_DOG
                .FI
                STA     WATCH_DOG_SAVE

; Clear memory
                LDX     #0              Clear zeropage memory
.CLR_MEM        STZ     0,X
                INX
                BNE     .CLR_MEM

; Initialize T1 as serial input
                LDA     #SIN_MODE1      Setup timer 1 as serial input at
                STA     T1_CONTROL1      9600 baud
                LDA     #SIN_MODE2
                STA     T1_CONTROL2
                LDA     #SIN_MODE3
                STA     T1_CONTROL3
                LDA     #PRESC_9600
                STA     T1_PRESC
                LDA     /PRESC_9600
                STA     T1_PRESC+1
                LDA     #8              Input 8 bits
                STA     T1_ACCU

; Initialize T2 as serial output
                LDA     #SOUT_MODE1     Setup timer as serial output at
                STA     T2_CONTROL1      9600 baud
                LDA     #SOUT_MODE2
                STA     T2_CONTROL2
                LDA     #SOUT_MODE3
                STA     T2_CONTROL3
                LDA     #PRESC_9600
                STA     T2_PRESC
                LDA     /PRESC_9600
                STA     T2_PRESC+1

; Initialize T3 as 5ms interval timer
                LDA     #TIME_MODE1     Setup timer 3 as a 5ms timer
                STA     T3_CONTROL1
                LDA     #TIME_MODE2
                STA     T3_CONTROL2
                LDA     #TIME_MODE3
                STA     T3_CONTROL3
                LDA     #TIMEOUT_5MS
                STA     T3_ACCU
                LDA     /TIMEOUT_5MS
                STA     T3_ACCU+1

; Setup interrupt controller
                LDA     #%1111.1110     Clear all interrupt request flags
                STA     INT_CONTROL
                LDA     #T1_PRIORITY+T2_PRIORITY Setup priority registers
                STA     INT_SOURCE01
                LDA     #T3_PRIORITY
                STA     INT_SOURCE23
                STZ     INT_SOURCE45
                LDA     #P87_PRIORITY
                STA     INT_SOURCE67

; Setup I/O ports
                LDA     #%1111.1111     Setup port 5 as all inputs
                STA     P5
                STA     P5_DIR

                LDA     #%1111.1111     Setup port 6 as all inputs
                STA     P6
                STA     P6_DIR

                LDA     #%1100.0000     Setup b7 and b6 of port 7 as outputs
                STA     P7
                LDA     #%0011.1111
                STA     P7_DIR

                LDA     #%0000.0001     Setup b0 of port 8 as output
                STA     P8
                LDA     #%1111.1110
                STA     P8_DIR

; Initialize SB-Bus system
                JSR     READ_INIT       Initialize SB-Bus address and model

                LDA     #$FF            Clear last send character
                STA     LAST_SEND
                LDA     #NOTHING_TO_REP Setup repeat pointer to nothing
                STA     REPEAT_CMD       to repeat error
                LDA     /NOTHING_TO_REP
                STA     REPEAT_CMD+1

                LDA     #1              Prepare read counter, otherwise
                STA     READ_COUNTER     the 1st reading will take too long
                STA     CLEAR_STAT      Clear statistics for the 1st time
                STA     HOLD_READINGS   Hold 1st reading

;---------------------------------------------------------------------------
; ROUTINE  :  MAIN
; FUNCTION :  Main program loop
;---------------------------------------------------------------------------

MAIN            LDX     #STACK_BEGIN    Restart stack pointer
                TXS

.MAIN_LOOP      STZ     STOP_ACT        Clear stop activities flag
                STZ     ERROR_COUNT     Reset error counter
                JSR     DO_EVENTS       Service others than SB-Bus
                JSR     POLL_SB         See if SB-Bus needs servicing
                BRA     .MAIN_LOOP

;---------------------------------------------------------------------------
; ROUTINE  :  DO_EVENTS
; FUNCTION :  Main service routines, excluding SB-Bus
;---------------------------------------------------------------------------

DO_EVENTS       JSR     WALK_THE_DOG    Walk the dog if it's home
                JSR     POLL_SOUT       See if something should be send
                RTS

;---------------------------------------------------------------------------
; ROUTINE  :  WALK_THE_DOG
; FUNCTION :  Trigger the DOG
;---------------------------------------------------------------------------

WALK_THE_DOG    LDA     WATCH_DOG_SAVE  Trigger it by writing the inverse
                EOR     #%1111.1111      of last time
                .DO     DOG_HOME        Only if we want the dog
                STA     WATCH_DOG
                .FI
                STA     WATCH_DOG_SAVE
                RTS

;---------------------------------------------------------------------------
; ROUTINE  :  IIC_START
; FUNCTION :  Start an I2C message
;---------------------------------------------------------------------------

IIC_START       LDA     P7              Make CLK and SDA high to be sure
                ORA     #%1000.0000
                STA     P7
                ORA     #%1100.0000
                STA     P7
                AND     #%0111.1111     Now make SDA low
                STA     P7
                STA     P7              (for delay)
                AND     #%0011.1111     Finally make SCL low too
                STA     P7
                RTS

;---------------------------------------------------------------------------
; ROUTINE  :  IIC_STOP
; FUNCTION :  Stop the I2C message
;---------------------------------------------------------------------------

IIC_STOP        LDA     P7              Make SDA low now
                AND     #%0111.1111      (CLK was already low)
                STA     P7
                ORA     #%0100.0000     Now make CLK high
                STA     P7
                STA     P7              (for delay)
                ORA     #%1100.0000     Now also make SDA high (STOP
                STA     P7
                RTS

;---------------------------------------------------------------------------
; ROUTINE  :  IIC_WRITE
; FUNCTION :  Write a byte to the I2C bus
;             No registers affected. C represents acknowledge level
;---------------------------------------------------------------------------

IIC_WRITE       PHA                     Save affected registers
                PHX
                PHY
                LDY     #8              Do 8 bits
                TAX                     Save byte to be sent
.LOOP           TXA                     Shift next bit to the carry
                ASL
                TAX
                LDA     P7              Get copy of current I/O situation
                AND     #%0111.1111     In case a 0 has to be sent
                BCC     .0              It must be a 0!
                ORA     #%1000.0000     Send a 1
.0              STA     P7              Copy bit to output
                ORA     #%0100.0000     Make CLK high for a few micro seconds
                STA     P7
                STA     P7
                AND     #%1011.1111     Then make it low again
                STA     P7
                DEY                     Do a total of 8 bits like this
                BNE     .LOOP           Not done yet!
                ORA     #%1000.0000     Make SDA high to accept the carry
                STA     P7
                ORA     #%0100.0000     Make acknowledge clock pulse high
                STA     P7
                LDX     P7              Read acknowledge bit
                AND     #%1011.1111     Finally make CLK low again
                STA     P7
                TXA                     Put acknowledge level in the carry
                ASL
                PLY                     Restore affected registers
                PLX
                PLA
                RTS

;---------------------------------------------------------------------------
; ROUTINE  :  IIC_READ
; FUNCTION :  Read a byte from the I2C bus
;             Accu holds result byte. C must reflect wanted ack. level
;---------------------------------------------------------------------------

IIC_READ        PHX                     Save affected registers
                PHP                     Save wanted acknowledge level
                LDX     #%0000.0001     Initialize result register
                LDA     P7              Make SDA line high to accept data
                ORA     #%1000.0000
                STA     P7
.LOOP           ORA     #%0100.0000     Make read CLK pulse high
                STA     P7
                LDA     P7              Read input bit
                ASL                      and put it in the carry
                TXA                     Then roll it into the result
                ROL
                TAX
                LDA     P7              Make CLK low again
                AND     #%1011.1111
                ORA     #%1000.0000     But enshure high level on SDA
                STA     P7
                BCC     .LOOP           Do until a 1 rolls out of result!
                PLP                     Send acknowledge?
                BCS     .NAK            No!
                AND     #%0111.1111     Make acknowledge pulse low
.NAK            STA     P7
                ORA     #%0100.0000     Send acknowledge CLK pulse
                STA     P7
                STA     P7
                AND     #%1011.1111
                STA     P7
                TXA                     Get value
                PLX
                RTS

;---------------------------------------------------------------------------
; ROUTINE  :  READ_NVM
; FUNCTION :  Read EEPROM data
;             Z=1 if EEPROM data valid
;---------------------------------------------------------------------------

READ_NVM        JSR     IIC_START       Start the IC message
                LDA     #EEPROM_WRITE   Send write address
                JSR     IIC_WRITE
                BCC     .ACK            Acknoweldge received!
                JSR     IIC_STOP        No one's listening, but still
                LDA     #%0000.0001     Set no acknowledge error
                STA     EEPROM_ERRORS
                RTS
.ACK            LDA     #0              Send cell address of 1st data
                JSR     IIC_WRITE
                JSR     IIC_START       Send new start address
                LDA     #EEPROM_READ    Send read address
                JSR     IIC_WRITE
                CLC                     Read a byte (with acknowledge)
                JSR     IIC_READ
                STA     PARAMETERS+1    Save it in parameter list
                CLC                     Read 2nd byte (with acknowledge)
                JSR     IIC_READ
                STA     PARAMETERS+2
                SEC                     Read last byte (wo acknowledge)
                JSR     IIC_READ
                STA     PARAMETERS+3
                JSR     IIC_STOP        Stop IC message
                LDA     PARAMETERS+1    See if check byte 1 matches
                EOR     PARAMETERS+2
                CMP     #%0101.0101
                BNE     .RTS            No! Invalid data
                LDA     PARAMETERS+1    See if check byte 2 matches too
                EOR     PARAMETERS+3
                CMP     #%1010.1010
.RTS            RTS                     Z=1 if data valid

;---------------------------------------------------------------------------
; ROUTINE  :  WRITE_NVM
; FUNCTION :  Write NVM data
;---------------------------------------------------------------------------

WRITE_NVM       STA     PARAMETERS+1    Send data byte to the EEPROM
                LDX     #0               to address 0
                JSR     .WRITE
                LDA     PARAMETERS+1    Send 1st check byte
                EOR     #%0101.0101
                LDX     #1
                JSR     .WRITE_BYTE
                LDA     PARAMETERS+1    Send 2nd check byte
                EOR     #%1010.1010
                LDX     #2
.WRITE_BYTE     PHA                     Save byte to be sent
                PHX                      and it's address
                LDA     #$FF            1st erase the byte
                JSR     .WRITE
                PLX                     Then write the data
                PLA
.WRITE          PHA                     Save data byte
                JSR     IIC_START       Start the IC message
                LDA     #EEPROM_WRITE   Send EEPROM write address
                JSR     IIC_WRITE
                TXA                     Send cell address
                JSR     IIC_WRITE
                PLA                     Then send data byte
                JSR     IIC_WRITE
                JSR     IIC_STOP        Stop the message
                LDY     #255            Wait until IC programmed
.BUSY           JSR     IIC_START       Start the IC message
                LDA     #EEPROM_WRITE   See if the EEPROM is still busy
                JSR     IIC_WRITE
                JSR     IIC_STOP
                DEY                     Decrement safety counter
                BEQ     .TIME_OUT       It took him too long!
                BCS     .BUSY           EEPROM is still busy
                RTS
.TIME_OUT       LDA     EEPROM_ERRORS   Is there an error already?
                BNE     .RTS            Yes! Don't change it
                LDA     #%0000.0010     Set unable to write EEPROM error
                STA     EEPROM_ERRORS    bit
.RTS            RTS

;---------------------------------------------------------------------------
; ROUTINE  :  READ_INIT
; FUNCTION :  Read initial values
;---------------------------------------------------------------------------

READ_INIT       JSR     READ_NVM        Get EEPROM contents
                BEQ     .MEM_OK         Memory's OK!
                JSR     READ_NVM        Try it again
                BEQ     .MEM_OK         Memroy's OK!
                LDA     EEPROM_ERRORS   Is there an error already?
                BNE     .ERROR          Yes! Let it be like that
                LDA     #3              Make it memory lost
                STA     EEPROM_ERRORS
.ERROR          LDA     #254            Set default SB-Bus address
                STA     MY_ADDRESS
                STZ     FLUKE_MODEL     Default model is 8010
                LDA     #254-128        Write these defaults to EEPROM
                JSR     WRITE_NVM
                RTS
.MEM_OK         LDA     PARAMETERS+1    Get my address and fluke model
                ORA     #%1000.0000
                STA     MY_ADDRESS
                LDA     PARAMETERS+1
                AND     #%1000.0000
                STA     FLUKE_MODEL
                RTS
